package com.jz.dm.common.enums;



import com.jz.dm.common.util.ResultCode;

import java.util.HashMap;
import java.util.Map;

/**
 * 网关结果码
 *
 */
public enum GatewayResultCode implements ResultCode {

    /** 处理成功 */
    SUCCESS("SUCCESS", "处理成功"),

    /** 处理成功 */
    FAIL("FAIL", "处理失败"),

    /** 未知异常 */
    UNKNOWN_EXCEPTION("UNKNOWN_EXCEPTION", "未知异常"),

    /** 无效接口 */
    ILLEGAL_INTERFACE("ILLEGAL_INTERFACE", "无效接口"),

    /** 无效参数 */
    ILLEGAL_ARGUMENT("ILLEGAL_ARGUMENT", "无效参数"),
    /** 参数列表为空 */
    REQUEST_PARAM_EMPTY("REQUEST_PARAM_EMPTY", "请求参数为空"),

    /** 无效请求 */
    ILLEGAL_REQUEST("ILLEGAL_REQUEST", "无效请求"),

    /** 无效请求 */
    ILLEGAL_TIMETEMP("ILLEGAL_TIMETEMP", "无效时间戳"),



    /** 请求次数受限 */
    REQUEST_LIMIT_EXCEPTION("REQUEST_LIMIT_EXCEPTION", "请求次数受限"),


    /** 请求超过限制 */
    RATE_LIMIT_EXCEEDED("RATE_LIMIT_EXCEEDED", "请求超过限制"),

    /** 无效路由类型 */
    ILLEGAL_ROUTE_TYPE("ILLEGAL_ROUTE_TYPE", "无效路由类型"),

    /** ip不允许访问 */
    IP_NOT_ALLOW_ACCESS("IP_NOT_ALLOW_ACCESS", "ip不允许访问"),

    /** 提交数据过大 */
    POST_DATA_TOO_LARGE("POST_DATA_TOO_LARGE", "提交数据过大"),
    /** 签名错误 */
    SIGN_ERROR("SIGN_EMPTY", "签名错误"),

    /** 请求未授权 */
    REQUEST_NOT_AUTH("REQUEST_NOT_AUTH", "请求未授权"),

    /** 请求组织不存在 */
    ORG_NOT_EXIST("ORG_NOT_EXIST", "请求组织不存在"),

    /** API状态异常 */
    API_STATUS_EXCEPTION("API_STATUS_EXCEPTION", "API状态异常"),

    /** 请求组织状态异常 */
    ORG_STATE_EXCEPTION("ORG_STATE_EXCEPTION", "请求组织状态异常"),
    /** 请求信息不存在 */
    REQUEST_INFO_UNEXIST("REQUEST_INFO_UNEXIST", "请求信息不存在"),

    /** 输出类型错误 */
    OUTPUT_TYPE_EXCEPTION("OUTPUT_TYPE_EXCEPTION", "输出类型错误!");
    /**
     * 初始化保存到map里方便根据code获取
     */
    private static Map<String, GatewayResultCode> RESULT_CODES = new HashMap<String, GatewayResultCode>();

    static {
        for (GatewayResultCode gatewayResultCode : GatewayResultCode.values()) {
            RESULT_CODES.put(gatewayResultCode.code, gatewayResultCode);
        }
    }

    /** 结果码 */
    private String code;

    /** 结果码信息 */
    private String msg;

    /**
     * 构造函数
     *
     * @param code       结果码
     * @param msg        结果码信息
     */
    private GatewayResultCode(String code, String msg) {
        this.code = code;
        this.msg = msg;
    }

    /**
     * 通过枚举<code>code</code>获得枚举
     *
     * @param   code    结果码
     * @return  枚举
     */
    public static GatewayResultCode getGatewayResultCode(String code) {
        return RESULT_CODES.get(code);
    }

    /**
     * Getter method for property <tt>code</tt>.
     *
     * @return property value of code
     */
    @Override
    public String getCode() {
        return code;
    }

    /**
     * Setter method for property <tt>code</tt>.
     *
     * @param code value to be assigned to property code
     */
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * Getter method for property <tt>msg</tt>.
     *
     * @return property value of msg
     */
    @Override
    public String getMsg() {
        return msg;
    }

    /**
     * Setter method for property <tt>msg</tt>.
     *
     * @param msg value to be assigned to property msg
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

}
