package com.jz.dm.common.exception;


import com.jz.dm.common.util.ResultCode;

/**
 * openapi异常
 *
 */
public class OpenApiException extends RuntimeException {

    private static final long serialVersionUID = -3963657380514719229L;

    /**
     * 结果码
     */
    private ResultCode resultCode;

    /**
     * 构造函数
     * 
     * @param resultCode
     */
    public OpenApiException(ResultCode resultCode) {
        this.resultCode = new ResultCodeImpl(resultCode);
    }

    /**
     * 构造函数
     * 
     * @param resultCode
     * @param cause
     */
    public OpenApiException(ResultCode resultCode, Throwable cause) {
        super(cause);
        this.resultCode = new ResultCodeImpl(resultCode);
    }

    /**
     * 构造函数
     * 
     * @param resultCode
     * @param detailMessage
     */
    public OpenApiException(ResultCode resultCode, String detailMessage) {
        super(detailMessage);
        this.resultCode = new ResultCodeImpl(resultCode);
    }

    /**
     * 构造函数
     * 
     * @param resultCode
     * @param detailMessage
     * @param cause
     */
    public OpenApiException(ResultCode resultCode, String detailMessage, Throwable cause) {
        super(detailMessage, cause);
        this.resultCode = new ResultCodeImpl(resultCode);
    }

    /**
     * 构造函数
     * 
     * @param code
     * @param msg
     */
    public OpenApiException(String code, String msg) {
        this(new ResultCodeImpl(code, msg));
    }

    /**
     * 构造函数
     * 
     * @param code
     * @param msg
     * @param cause
     */
    public OpenApiException(String code, String msg, Throwable cause) {
        this(new ResultCodeImpl(code, msg), cause);
    }

    /**
     * 构造函数
     * 
     * @param code
     * @param msg
     * @param detailMessage
     */
    public OpenApiException(String code, String msg, String detailMessage) {
        this(new ResultCodeImpl(code, msg), detailMessage);
    }

    /**
     * Getter method for property <tt>resultCode</tt>.
     * 
     * @return property value of resultCode
     */
    public ResultCode getResultCode() {
        return resultCode;
    }

    public static class ResultCodeImpl implements ResultCode {

        private String code;

        private String msg;

        public ResultCodeImpl(ResultCode resultCode) {
            this.code = resultCode.getCode();
            this.msg = resultCode.getMsg();
        }

        public ResultCodeImpl(String code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        @Override
        public String getCode() {
            return code;
        }

        @Override
        public String getMsg() {
            return msg;
        }

    }

}
