package com.jz.dm.common.util;

import org.slf4j.Logger;

/**
 * 日志打印工具
 *
 */
public class LogUtil {

    /** 摘要日志的内容分隔符 */
    public static final String SEP       = ",";

    /** 修饰符 */
    private static final char  RIGHT_TAG = ']';

    /** 修饰符 */
    private static final char  LEFT_TAG  = '[';

    /**
     * 日志信息
     *
     */
    public interface LogInfo {
        /**
         * 获取日志信息
         * 
         * @return 日志信息
         */
        String getLogInfo();
    }

    /**
     * 打印info日志。
     * 
     * @param logger    日志对象
     * @param objs      任意个要输出到日志的参数
     */
    public static void info(Logger logger, Object... objs) {
        if (logger.isInfoEnabled()) {
            logger.info(getLogString(objs));
        }
    }

    /**
     * 打印info级别日志
     * 
     * @param logger 日志对象
     * @param logInfo 日志信息
     */
    public static void info(Logger logger, LogInfo logInfo) {
        if (logger.isInfoEnabled()) {
            logger.info(logInfo.getLogInfo());
        }
    }

    /**
     * 打印info级别日志
     * 
     * @param logger    日志对象
     * @param e         异常信息
     * @param objs      任意个要输出到日志的参数
     */
    public static void info(Logger logger, Throwable e, Object... objs) {
        if (logger.isInfoEnabled()) {
            logger.info(getLogString(objs), e);
        }
    }

    /**
     * 打印warn级别日志
     * 
     * @param logger    日志对象
     * @param objs      任意个要输出到日志的参数
     */
    public static void warn(Logger logger, Object... objs) {
        logger.warn(getLogString(objs));
    }

    /**
     * 打印warn级别日志
     * 
     * @param logger    日志对象
     * @param e         异常信息
     * @param objs      任意个要输出到日志的参数
     */
    public static void warn(Logger logger, Throwable e, Object... objs) {
        logger.warn(getLogString(objs), e);
    }

    /**
     * 打印error级别日志
     * 
     * @param logger    日志对象
     * @param e         异常信息
     * @param objs      任意个要输出到日志的参数
     */
    public static void error(Logger logger, Throwable e, Object... objs) {
        logger.error(getLogString(objs), e);
    }

    /**
     * 打印error级别日志
     * 
     * @param logger    日志对象
     * @param objs      任意个要输出到日志的参数
     */
    public static void error(Logger logger, Object... objs) {
        logger.error(getLogString(objs));
    }

    /**
     * 打印debug日志。
     * 
     * @param logger    日志对象
     * @param objs      任意个要输出到日志的参数
     */
    public static void debug(Logger logger, Object... objs) {
        if (logger.isDebugEnabled()) {
            logger.debug(getLogString(objs));
        }
    }

    /**
     * 打印debug日志。
     * 
     * @param logger    日志对象
     * @param e         异常信息
     * @param objs      任意个要输出到日志的参数
     */
    public static void debug(Logger logger, Throwable e, Object... objs) {
        if (logger.isDebugEnabled()) {
            logger.debug(getLogString(objs), e);
        }
    }

    /**
     * 生成输出到日志的字符串
     * 
     * @param objs      任意个要输出到日志的参数
     * @return          日志字符串
     */
    public static String getLogString(Object... objs) {
        StringBuilder log = new StringBuilder();
        log.append(LEFT_TAG).append(getThreadId()).append(RIGHT_TAG);
        for (Object o : objs) {
            log.append(o).append(SEP);
        }
        return log.toString();
    }

    /**
     * 获取线程id。
     * 
     * @return 线程id
     */
    public static String getThreadId() {
        return String.valueOf(Thread.currentThread().getId());
    }
}
