package com.jz.dm.common.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

import java.util.HashMap;
import java.util.Map;

/**
 * openapi响应
 *
 */
public class OpenApiResponse {

    /**
     * 结果码
     */
    private static final String       ATTRIBUTE_NAME_CODE = "code";

    /**
     * 结果消息
     */
    private static final String       ATTRIBUTE_NAME_MSG  = "msg";

    /**
     * 属性
     */
    private final Map<String, Object> attributes          = new HashMap<String, Object>();

    /**
     * 获取结果码
     * 
     * @return
     */
    public String getCode() {
        return (String) attributes.get(ATTRIBUTE_NAME_CODE);
    }

    /**
     * 设置结果码
     * 
     * @param code
     */
    public void setCode(String code) {
        attributes.put(ATTRIBUTE_NAME_CODE, code);
    }

    /**
     * 获取结果信息
     * 
     * @return
     */
    public String getMsg() {
        return (String) attributes.get(ATTRIBUTE_NAME_MSG);
    }

    /**
     * 设置结果信息
     * 
     * @param msg
     */
    public void setMsg(String msg) {
        attributes.put(ATTRIBUTE_NAME_MSG, msg);
    }

    /**
     * 获取属性
     * 
     * @param attributeName
     * @return
     */
    @SuppressWarnings("unchecked")
    public <T> T getAttribute(String attributeName) {
        return (T) attributes.get(attributeName);
    }

    /**
     * 设置属性
     * 
     * @param attributeName
     * @param attributeValue
     */
    public void setAttribute(String attributeName, Object attributeValue) {
        attributes.put(attributeName, attributeValue);
    }

    /**
     * 设置对象属性
     * 
     * @param object
     */
    public void setAttribute(Object object) {
        Object jsonObject = JSON.toJSON(object);
        if (!(jsonObject instanceof JSONObject)) {
            throw new IllegalArgumentException("object must be a javabean.");
        }
        attributes.putAll((JSONObject) jsonObject);
    }

    /**
     * 获取所有属性
     * 
     * @return
     */
    public Map<String, Object> getAttributes() {
        return attributes;
    }

}
