package com.jz.dm.common.util;

import java.util.HashMap;
import java.util.Map;

/**
 * 结果码
 *
 */
public enum OpenApiResultCode implements ResultCode {

    /** 处理成功 */
    SUCCESS("SUCCESS", "处理成功"),

    /** 未知异常 */
    UNKNOWN_EXCEPTION("UNKNOWN_EXCEPTION", "未知异常"),

    /** 无效接口 */
    ILLEGAL_INTERFACE("ILLEGAL_INTERFACE", "无效接口"),

    /** 无效参数 */
    ILLEGAL_ARGUMENT("ILLEGAL_ARGUMENT", "无效参数"),

    /** 名类型不支持 */
    SIGN_TYPE_NOT_SUPPORT("SIGN_TYPE_NOT_SUPPORT", "签名类型不支持"),

    /** 数据签名错误 */
    DATA_SIGN_ERROR("DATA_SIGN_ERROR", "数据签名错误"),

    /** 公钥格式错误 */
    PUBLIC_KEY_FORMAT_ERROR("PUBLIC_KEY_FORMAT_ERROR", "公钥格式错误"),

    /** 私钥格式错误 */
    PRIVATE_KEY_FORMAT_ERROR("PRIVATE_KEY_FORMAT_ERROR", "私钥格式错误"),

    /** 响应数据格式错误 */
    RESPONSE_DATA_FORMAT_ERROR("RESPONSE_DATA_FORMAT_ERROR", "响应数据格式错误"),

    /** 签名校验错误 */
    SIGN_VERIFY_ERROR("SIGN_VERIFY_ERROR", "签名校验错误"),

    /** 不支持该信息摘要算法 */
    NO_SUCH_MD_ALGORITHM("NO_SUCH_MD_ALGORITHM", "不支持该信息摘要算法"),

    /** 信息摘要错误 */
    MESSAGE_DIGEST_ERROR("MESSAGE_DIGEST_ERROR", "信息摘要错误"),

    /** 数据加密错误 */
    DATA_ENCRYPTION_ERROR("DATA_ENCRYPTION_ERROR", "数据加密错误"),;

    /**
     * 初始化保存到map里方便根据code获取
     */
    private static Map<String, OpenApiResultCode> RESULT_CODES = new HashMap<String, OpenApiResultCode>();

    static {
        for (OpenApiResultCode openApiCode : OpenApiResultCode.values()) {
            RESULT_CODES.put(openApiCode.code, openApiCode);
        }
    }

    /** 结果码 */
    private String code;

    /** 结果码信息 */
    private String msg;

    /**
     * 构造函数
     *
     * @param code       结果码
     * @param msg        结果码信息
     */
    private OpenApiResultCode(String code, String msg) {
        this.code = code;
        this.msg = msg;
    }

    /**
     * 通过枚举<code>code</code>获得枚举
     *
     * @param   code    结果码
     * @return  枚举
     */
    public static OpenApiResultCode getOpenApiResultCode(String code) {
        return RESULT_CODES.get(code);
    }

    /**
     * Getter method for property <tt>code</tt>.
     *
     * @return property value of code
     */
    @Override
    public String getCode() {
        return code;
    }

    /**
     * Setter method for property <tt>code</tt>.
     *
     * @param code value to be assigned to property code
     */
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * Getter method for property <tt>msg</tt>.
     *
     * @return property value of msg
     */
    @Override
    public String getMsg() {
        return msg;
    }

    /**
     * Setter method for property <tt>msg</tt>.
     *
     * @param msg value to be assigned to property msg
     */
    public void setMsg(String msg) {
        this.msg = msg;
    }

}
