package com.jz.dm.common.util;

import com.jz.dm.common.constant.Constants;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.UnsupportedEncodingException;
import java.util.Calendar;
import java.util.Random;
import java.util.regex.Pattern;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.common.util
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: RandomUtil
 * @DATE: 2020-12-30/9:50
 * @DAY_NAME_SHORT: 周三
 * @Description: 生成随机编码工具类
 **/
@Slf4j
public class RandomUtil {
    private final static int[] li_SecPosValue = {1601, 1637, 1833, 2078, 2274,
            2302, 2433, 2594, 2787, 3106, 3212, 3472, 3635, 3722, 3730, 3858,
            4027, 4086, 4390, 4558, 4684, 4925, 5249, 5590};
    private final static String[] lc_FirstLetter = {"a", "b", "c", "d", "e",
            "f", "g", "h", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s",
            "t", "w", "x", "y", "z"};

    /**
     * 生成指定长度字母+数字的随机数
     *
     * @param length 长度
     * @return
     */
    public static String getStringRandom(int length) {
        String val = "";
        Random random = new Random();
        //参数length，表示生成几位随机数
        for (int i = 0; i < length; i++) {
            String charOrNum = random.nextInt(2) % 2 == 0 ? "char" : "num";
            //输出字母还是数字
            if ("char".equalsIgnoreCase(charOrNum)) {
                //输出是大写字母还是小写字母
                int temp = random.nextInt(2) % 2 == 0 ? 65 : 97;
                val += (char) (random.nextInt(26) + temp);
            } else if ("num".equalsIgnoreCase(charOrNum)) {
                val += String.valueOf(random.nextInt(10));
            }
        }
        log.info("生成指定长度字母+数字的随机数为:{}", val);
        return val;
    }

    /**
     * 生成组织(内/外)部编码
     *
     * @param type    类型
     * @param content
     * @return
     */
    public static String generateOrgCoding(String content, String type) {
        String coding = "";
        if (null == content || null == type) {
            log.info("----------生成组织编码内容或类型不存在--------------");
            return "";
        }
     /*   boolean chineseWord = isChineseWord(content);
        if (!chineseWord) {
            log.info("----------编码中不能出现英文或者字母--------------");
            return "";
        }*/
        StringBuilder builder = new StringBuilder();
        if (Constants.AUTH_INT.equalsIgnoreCase(type)) {
            builder.append("NEI");
        } else {
            builder.append("WAI");
        }
        String orgContent = getAllFirstLetter(content);
        orgContent = orgContent.toUpperCase();
        builder.append(orgContent);
        if (builder.length() > 12) {
            coding = builder.toString().substring(0, 13);
        } else if (builder.length() == 12) {
            coding = builder.toString();
        } else if (builder.length() < 12) {
            String random = getStringRandom(12 - builder.length());
            builder.append(random);
            coding = builder.toString();
            log.info("-----------生成的组织编码为:{}", coding);
        }
        return coding;
    }

    /**
     * 生成内/外部授权码
     *
     * @param orgCode
     * @param userId
     * @param type
     * @return
     */
    public static String generateTokenCode(String type, String orgCode, String userId) {
        String coding = "";
        StringBuilder builder = new StringBuilder();
        Calendar cal = Calendar.getInstance();
        int year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH) + 1;
        int day = cal.get(Calendar.DAY_OF_MONTH);
        int hour = cal.get(Calendar.HOUR_OF_DAY);
        int minute = cal.get(Calendar.MINUTE);
        int second = cal.get(Calendar.SECOND);
        builder.append(year);//年
        if (StringUtils.isNotBlank(type) && Constants.AUTH_INT.equalsIgnoreCase(type)) {//内部授权码
            if (StringUtils.isBlank(orgCode)) {
                return "";
            }
            coding = getCodeInfo(orgCode, coding, builder, month, day, hour, minute, second);
        } else if (StringUtils.isNotBlank(type) && Constants.AUTH_OUT.equalsIgnoreCase(type)) { //外部组织
            if (StringUtils.isBlank(userId)) {
                return "";
            }
            if (userId.length() < 8) { //补全用户id
                String userNum = addZeroForNum(userId, 8);
                builder.append(userNum);
            }
            coding = getCodeInfo(userId, coding, builder, month, day, hour, minute, second);
        }
        return coding;
    }

    /**
     * 拼接内部/外部授权码
     *
     * @param orgCode
     * @param coding
     * @param builder
     * @param month
     * @param day
     * @param hour
     * @param minute
     * @param second
     * @return
     */
    private static String getCodeInfo(String orgCode, String coding,
                                      StringBuilder builder, int month,
                                      int day, int hour, int minute, int second) {
        builder.append(orgCode);//组织编码
        builder.append(month);//月
        builder.append(day);//日
        builder.append(hour);//时
        builder.append(minute);//分
        builder.append(second);//秒
        if (builder.length() > 32) {
            coding = builder.toString().substring(0, 33);
        } else if (builder.length() == 32) {
            coding = builder.toString();
        } else if (builder.length() < 32) {
            String random = getStringRandom(32 - builder.length());
            builder.append(random);
            coding = builder.toString();
            log.info("-----------生成的内部授权码为:{}", coding);
        }
        return coding;
    }

    /**
     * 取得给定汉字串的首字母串,即声母串
     *
     * @param str 给定汉字串
     * @return 声母串
     */
    public static String getAllFirstLetter(String str) {
        if (str == null || str.trim().length() == 0) {
            return "";
        }
        String _str = "";
        for (int i = 0; i < str.length(); i++) {
            _str = _str + getFirstLetter(str.substring(i, i + 1));
        }
        return _str;
    }

    /**
     * 取得给定汉字的首字母,即声母
     *
     * @param chinese 给定的汉字
     * @return 给定汉字的声母
     */
    public static String getFirstLetter(String chinese) {
        if (chinese == null || chinese.trim().length() == 0) {
            return "";
        }
        chinese = conversionStr(chinese, "GB2312", "ISO8859-1");
        if (chinese.length() > 1) { // 判断是不是汉字
            int li_SectorCode = (int) chinese.charAt(0); // 汉字区码
            int li_PositionCode = (int) chinese.charAt(1); // 汉字位码
            li_SectorCode = li_SectorCode - 160;
            li_PositionCode = li_PositionCode - 160;
            int li_SecPosCode = li_SectorCode * 100 + li_PositionCode; // 汉字区位码
            if (li_SecPosCode > 1600 && li_SecPosCode < 5590) {
                for (int i = 0; i < 23; i++) {
                    if (li_SecPosCode >= li_SecPosValue[i]
                            && li_SecPosCode < li_SecPosValue[i + 1]) {
                        chinese = lc_FirstLetter[i];
                        break;
                    }
                }
            } else // 非汉字字符,如图形符号或ASCII码
            {
                chinese = conversionStr(chinese, "ISO8859-1", "GB2312");
                chinese = chinese.substring(0, 1);
            }
        }
        return chinese;
    }

    /**
     * 字符串编码转换
     *
     * @param str           要转换编码的字符串
     * @param charsetName   原来的编码
     * @param toCharsetName 转换后的编码
     * @return 经过编码转换后的字符串
     */
    private static String conversionStr(String str, String charsetName, String toCharsetName) {
        try {
            str = new String(str.getBytes(charsetName), toCharsetName);
        } catch (UnsupportedEncodingException ex) {
            System.out.println("字符串编码转换异常：" + ex.getMessage());
        }
        return str;
    }

    /**
     * 限制只能输入中文
     */
    public static boolean isChineseWord(String str) {
        String pattern = "[\u4e00-\u9fa5]+";
        boolean isMatch = Pattern.matches(pattern, str);
        return isMatch;
    }

    /**
     * 生成8位用户id,不够的前面用0补全
     *
     * @param str
     * @param strLength
     * @return
     */
    public static String addZeroForNum(String str, int strLength) {
        int strLen = str.length();
        if (strLen < strLength) {
            while (strLen < strLength) {
                StringBuffer sb = new StringBuffer();
                sb.append("0").append(str);// 左补0
                // sb.append(str).append("0");//右补0
                str = sb.toString();
                strLen = str.length();
            }
        }
        return str;
    }


    /**
     * 生成指定版本号
     * @param version 旧版本号
     * @return
     */
    public static String generateVersion(String version) {
        StringBuilder builder = new StringBuilder();
        if (StringUtils.isNotBlank(version)) {
            String[] split  = version.substring(1).split("\\.");
            Integer one = Integer.parseInt(split[0]);
            Integer two = Integer.parseInt(split[1]);
            if (two + 1 == 100) {
                one = one + 1;
                two = 0;
            } else if (two < 100) {
                two = two + 1;
            }
            builder.append("v");
            builder.append(one.toString());
            builder.append(".");
            builder.append(two.toString());
        }else {
            return "";
        }
       return builder.toString();
    }

    public static void main(String[] args) {
        //System.out.println(getStringRandom(8));
        // System.out.println("获取拼音首字母："+getAllFirstLetter("大中国南昌中大china"));
        //System.out.println(isChineseWord("大中国南昌中大"));
        //System.out.println(generateOrgCoding("九章数据","INT"));
        // System.out.println(generateTokenCode("", ""));
        //System.out.println(addZeroForNum("1", 8));
        //System.out.println("内部授权码:" + generateTokenCode("INT", "NEIJZSJ900hx", ""));//内部授权码
        //System.out.println("外部授权码:" + generateTokenCode("OUT", "", "199"));//内部授权码
        System.out.println(generateVersion("v1.99"));
    }
}
