package com.jz.dm.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.PropertyNamingStrategy;
import com.jz.dm.common.util.JsonUtil;
import com.jz.dm.gateway.GatewayService;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import com.jz.dm.models.enity.RequestContext;
import io.swagger.annotations.Api;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * 网关请求
 * @author zc
 */
@RestController
@RequestMapping("api/")
@Api(tags = {"api请求controller"})
public class GatewayController {

    @Autowired
    private GatewayService gatewayService;

    @RequestMapping(value = "/gateway", consumes = "application/json")
    public String gateway(@RequestBody String json, HttpServletRequest httpServletRequest,
                          HttpServletResponse httpServletResponse) {
        GatewayRequest gatewayRequest = JSON.parseObject(json, GatewayRequest.class);
        return doGateway(gatewayRequest, httpServletRequest, httpServletResponse);
    }

    @RequestMapping(value = "/gateway")
    public String gateway(GatewayRequest gatewayRequest, HttpServletRequest httpServletRequest,
                          HttpServletResponse httpServletResponse) {
        return doGateway(gatewayRequest, httpServletRequest, httpServletResponse);
    }

    private String doGateway(GatewayRequest gatewayRequest, HttpServletRequest httpServletRequest,
                             HttpServletResponse httpServletResponse) {
        try {
            RequestContext requestContext = RequestContext.getCurrentContext();
            requestContext.setRequest(httpServletRequest);
            requestContext.setResponse(httpServletResponse);
            GatewayResponse gatewayResponse = gatewayService.invoke(gatewayRequest);
            JSONObject result = new JSONObject();
            convertResponse(result, gatewayResponse);
            return JsonUtil.toJSONString(result, PropertyNamingStrategy.SnakeCase);
        } finally {
            RequestContext.getCurrentContext().reset();
        }
    }

    private void convertResponse(JSONObject result, GatewayResponse gatewayResponse) {
        result.put("code", gatewayResponse.getCode());
        result.put("msg", gatewayResponse.getMsg());
        Map<String, Object> data = gatewayResponse.getResponse();
        data.remove("code");
        data.remove("msg");
        if (!CollectionUtils.isEmpty(data)) {
            result.put("data", data);
            if (StringUtils.isNotBlank(gatewayResponse.getEncryptData())) {
                result.put("data", gatewayResponse.getEncryptData());
            }
        }
    }
}
