package com.jz.dm.filter;

import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import com.jz.dm.models.enity.RequestContext;
import com.jz.dm.common.enums.GatewayResultCode;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;

/**
 * 检查提交数据大小
 * @author key
 */
@Component("CheckPostSizeFilter")
public class CheckPostSizeFilter extends AbstractFilter {


    /**
     * 过滤器名称
     * @return
     */
    @Override
    public String getFilterName() {
        return "CheckPostSizeFilter";
    }


    /** 
     * 过滤器执行顺序
     */
    @Override
    public int getOrder() {
        return Constants.FILTER_ORDER_1;
    }

    @Override
    protected void internalDoFilter(GatewayRequest request, GatewayResponse response,
                                    FilterChain chain) {
        HttpServletRequest httpServletRequest = RequestContext.getCurrentContext().getRequest();
        Boolean postDataTooLarge = (Boolean) httpServletRequest.getAttribute("postDataTooLarge");
        if (postDataTooLarge != null && postDataTooLarge) {
            Integer maxPostSize = (Integer) httpServletRequest.getAttribute("maxPostSize");
            Integer postSize = (Integer) httpServletRequest.getAttribute("postSize");
            throw new GatewayException(GatewayResultCode.POST_DATA_TOO_LARGE,
                "maxPostSize:" + maxPostSize + ", postSize:" + postSize);
        }
        chain.doFilter(request, response);
    }

}
