package com.jz.dm.filter;

import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import com.jz.dm.common.enums.GatewayResultCode;
import org.springframework.stereotype.Component;

/**
 * 时间戳校验过滤器
 *
 * @author key
 */
@Component
public class CheckTimestampFilter extends AbstractFilter {

    /**
     * 时间戳超时分钟，10分钟
     */
    //private static final long max = 1000 * 60 * 10; //10分钟
    private static final long max = 1000 * 60 * 60*24; //24小时

    @Override
    public int getOrder() {
        return Constants.FILTER_ORDER_4;
    }

    @Override
    public String getFilterName() {
        return "CheckTimestampFilter";
    }


    @Override
    protected void internalDoFilter(GatewayRequest request, GatewayResponse response, FilterChain chain) {
            String timestamp = request.getTimestamp();
            long time = 0;
            try {
                time = Long.valueOf(timestamp);
            } catch (Exception ex) {
                throw new GatewayException(GatewayResultCode.ILLEGAL_ARGUMENT);//无效参数
            }
            if (System.currentTimeMillis() - time > max) {
                throw new GatewayException(GatewayResultCode.ILLEGAL_TIMETEMP);//无效时间戳
            }
        chain.doFilter(request, response);
    }


}
