package com.jz.dm.filter;

import com.jz.dm.common.exception.OpenApiException;
import com.jz.dm.common.util.LogUtil;
import com.jz.dm.common.util.ResultCode;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import com.jz.dm.common.enums.GatewayResultCode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 过滤链
 *
 * @author key
 */
public class FilterChainImpl implements FilterChain {

    private final Logger LOGGER  = LoggerFactory.getLogger(this.getClass());

    private int          pos     = 0;

    private final List<Filter> filters = new ArrayList<Filter>();

    public FilterChainImpl() { }

    public FilterChainImpl(List<Filter> filters) {
        addFilters(filters);
    }

    @Override
    public void doFilter(GatewayRequest request, GatewayResponse response) {
        Filter nextFilter = nextFilter();
        if (nextFilter == null) {
            return;
        }
        try {
            nextFilter.doFilter(request, response, this);
        } catch (OpenApiException ex) {
            ResultCode resultCode = ex.getResultCode();
            LogUtil.error(LOGGER, ex, "doFilter occur exception，code=" + resultCode.getCode()
                                      + ",msg=" + resultCode.getMsg() + ",request=" + request);
            response.setCode(ex.getResultCode().getCode());
            response.setMsg(ex.getResultCode().getMsg());
        } catch (Throwable ex) {
            LogUtil.error(LOGGER, ex, "doFilter occur unknown exception，request=" + request);
            response.setCode(GatewayResultCode.UNKNOWN_EXCEPTION.getCode());
            response.setMsg(GatewayResultCode.UNKNOWN_EXCEPTION.getMsg());
        }
    }

    public Filter nextFilter() {
        if (pos < filters.size()) {
            return filters.get(pos++);
        }
        return null;
    }

    public void addFilters(List<Filter> filters) {
        this.filters.addAll(filters);
    }

    public void addFilter(Filter filter) {
        this.filters.add(filter);
    }

    public void addFilter(int index, Filter filter) {
        this.filters.add(index, filter);
    }

}
