package com.jz.dm.filter;

import com.alibaba.fastjson.JSONObject;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.common.constant.LoggingConstants;
import com.jz.dm.common.enums.GatewayResultCode;
import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.util.MapUtil;
import com.jz.dm.common.util.Md5;
import com.jz.dm.models.domian.ApiAuth;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import com.jz.dm.service.ApiInterfaceService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.TreeMap;

/**
 * 验签过滤器(验证签名信息)
 */
@Slf4j
@Component
public class VerifySignFilter extends AbstractFilter {

    @Autowired
    private ApiInterfaceService apiInterfaceService;
    @Override
    public int getOrder() {
        return Constants.FILTER_ORDER_3;
    }

    @Override
    public String getFilterName() {
        return "VerifySignFilter";
    }
    @Override
    protected void internalDoFilter(GatewayRequest request, GatewayResponse response,
                                    FilterChain chain) {
        try {
            TreeMap<String, Object> map = MapUtil.objToMap(request);
            if (null == map){
                log.info("~~~~~~~~~~~~~~~签名数据转换map异常~~~~~~~~~~~~~~~~~~");
                throw new GatewayException(GatewayResultCode.UNKNOWN_EXCEPTION);
            }
            List signParams = MapUtil.directSort(map);
            if (CollectionUtils.isNotEmpty(signParams)){
                JSONObject jsonObject = JSONObject.parseObject(request.getParams());
                //需要传入授权码
                ApiAuth apiAuthInfo = apiInterfaceService.getApiAuthInfo(request.getApiKey(), jsonObject.getString("authCode"));
                if (null == apiAuthInfo){
                    throw new GatewayException(GatewayResultCode.ILLEGAL_REQUEST);
                }
                String paramStr = MapUtil.listToString(signParams, LoggingConstants.AND_SPILT);
                String salt = Md5.encrypt(paramStr, apiAuthInfo.getSalt());
                if (!request.getSign().equals(salt)){
                    throw new GatewayException(GatewayResultCode.SIGN_ERROR);
                }
            }
        } catch(Exception e) {
            log.error("~~~~~~~~~~~~~~签名过滤异常~~~~~~~~~~~~~~~~~~");
            log.error("异常信息:{}",e.getMessage());
        }
        // 后期再扩展
        //OpenApi openApi = new OpenApi();
        //openApi.setApplication("JZ_API_GATEWAY");//应用
        //openApi.setRouteType(RouteType.SRPING);//漏油类型
        //RequestContext.getCurrentContext().set("openApi", openApi);

        chain.doFilter(request, response);
    }

}
