package com.jz.dm.models.enity;

import lombok.Data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * Dispatch Context
 */
@Data
public class DispatchContext implements Serializable {

    /**
     * 序列号
     */
    private static final long         serialVersionUID = -3448256355423553719L;

    /**
     * api唯一标识
     */
    private String                    appKey;

    /**
     * openapi接口
     */
    private String                    openApiMethod;

    /**
     * openapi接口参数，json格式
     */
    private String                    openApiParams;

    /**
     * openapi接口版本
     */
    private String                    openApiVersion;

    /**
     * 扩展属性
     */
    private final Map<String, Object> extAttributes    = new HashMap<String, Object>();


    /**
     * 获取扩展属性
     * 
     * @param attributeName
     * @return
     */
    @SuppressWarnings("unchecked")
    public <T> T getExtAttribute(String attributeName) {
        return (T) extAttributes.get(attributeName);
    }

    /**
     * 设置属性
     * 
     * @param attributeName
     * @param attributeValue
     */
    public void setExtAttribute(String attributeName, Object attributeValue) {
        extAttributes.put(attributeName, attributeValue);
    }

    /**
     * Getter method for property <tt>extAttributes</tt>.
     * 
     * @return property value of extAttributes
     */
    public Map<String, Object> getExtAttributes() {
        return extAttributes;
    }

    /** 
     * @see Object#toString()
     */
    @Override
    public String toString() {
        return "DispatchContext [openApiMethod=" + openApiMethod
               + ", openApiParams=" + openApiParams + ", openApiVersion=" + openApiVersion + "]";
    }

}
