package com.jz.dm.models.enity;


import com.jz.dm.web.annotation.ParamName;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 *  api请求
 * @author zc
 */
@ApiModel("api请求实体类")
@Data
public class GatewayRequest implements Serializable {

    @ParamName("api_key")
    @ApiModelProperty(value = "appKey API唯一标识")
    private String   apiKey;

    @ApiModelProperty(value = "接口名称")
    private String   method;

    @ApiModelProperty(value = "格式，目前仅支持JSON")
    private String   format;

    /**
     * 请求使用的编码格式，如UTF-8,GBK,GB2312等
     */
    //private String  charset;

    @ParamName("sign_type")
    @ApiModelProperty(value = "生成签名字符串所使用的签名算法类型")
    private String    signType;

    @ApiModelProperty(value = "请求参数的签名串")
    private String    sign;

    @ApiModelProperty(value = "发送请求的时间，格式yyyy-MM-dd HH:mm:ss")
    private String    timestamp;

    @ApiModelProperty(value="调用的接口版本，固定为:1.0")
    private String    version;

    @ApiModelProperty(value="请求参数,JSON格式")
    private String   params;

    @ApiModelProperty(value="扩展属性")
    private final Map<String, Attribute> extAttributes = new HashMap<String, Attribute>();

    /**
     * 获取扩展属性
     * 
     * @param attributeName
     * @return
     */
    public Attribute getExtAttribute(String attributeName) {
        return extAttributes.get(attributeName);
    }

    /**
     * 设置属性
     * 
     * @param attribute
     */
    public void setExtAttribute(Attribute attribute) {
        if (attribute == null) {
            return;
        }
        extAttributes.put(attribute.name, attribute);
    }

    /**
     * Getter method for property <tt>extAttributes</tt>.
     * 
     * @return property value of extAttributes
     */
    public Map<String, Attribute> getExtAttributes() {
        return extAttributes;
    }

    /** 
     * @see Object#toString()
     */
    @Override
    public String toString() {
        return "GatewayRequest [appKey=" + apiKey + ", method=" + method + ", format=" + format
               + ", signType=" + signType + ", sign=" + sign
               + ", timestamp=" + timestamp + ", version=" + version
               + ", params=" + params + ", extAttributes="
               + extAttributes + "]";
    }

    public static class Attribute {

        /**
         *  熟悉名称
         */
        public String  name;

        /**
         * 属性值
         */
        public Object  value;

        /**
         * 是否传递到业务系统
         */
        public boolean isPass;

        public Attribute(String name, Object value) {
            this(name, value, false);
        }

        public Attribute(String name, Object value, boolean isPass) {
            this.name = name;
            this.value = value;
            this.isPass = isPass;
        }

        /**
         * @see Object#toString()
         */
        @Override
        public String toString() {
            return "Attribute [name=" + name + ", value=" + value + ", isPass=" + isPass + "]";
        }

    }

}
