package com.jz.dm.models.enity;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 请求上下文
 *
 */
public class RequestContext extends ConcurrentHashMap<String, Object> {

    private static final long                                    serialVersionUID = -1993922957778726516L;

    protected static Class<? extends RequestContext>             CONTEXT_CLASS    = RequestContext.class;

    protected static final ThreadLocal<? extends RequestContext> THREAD_LOCAL     =                       //
            new ThreadLocal<RequestContext>() {
                @Override
                protected RequestContext initialValue() {
                    try {
                        return CONTEXT_CLASS.newInstance();
                    } catch (Throwable e) {
                        throw new RuntimeException(e);
                    }
                }
            };

    public RequestContext() {
        super();
    }

    /**
     * Override the default RequestContext
     *
     * @param clazz
     */
    public static void setContextClass(Class<? extends RequestContext> clazz) {
        CONTEXT_CLASS = clazz;
    }

    /**
     * Get the current RequestContext
     *
     * @return the current RequestContext
     */
    public static RequestContext getCurrentContext() {
        return THREAD_LOCAL.get();
    }

    /**
     * puts the key, value into the map. a null value will remove the key from the map
     *
     * @param key
     * @param value
     */
    public void set(String key, Object value) {
        if (value != null) {
            put(key, value);
        } else {
            remove(key);
        }
    }

    /**
     * @return the HttpServletRequest from the "request" key
     */
    public HttpServletRequest getRequest() {
        return (HttpServletRequest) get("request");
    }

    /**
     * sets the HttpServletRequest into the "request" key
     *
     * @param request
     */
    public void setRequest(HttpServletRequest request) {
        put("request", request);
    }

    /**
     * @return the HttpServletResponse from the "response" key
     */
    public HttpServletResponse getResponse() {
        return (HttpServletResponse) get("response");
    }

    /**
     * sets the "response" key to the HttpServletResponse passed in
     *
     * @param response
     */
    public void setResponse(HttpServletResponse response) {
        set("response", response);
    }

    /**
     * get exclude filter
     * 
     * @return
     */
    @SuppressWarnings("unchecked")
    public Set<String> getExcludeFilters() {
        return (Set<String>) get("excludeFilters");
    }

    /**
     * set exclude filter
     * 
     * @param excludeFilters
     */
    public void setExcludeFilters(Set<String> excludeFilters) {
        set("excludeFilters", excludeFilters);
    }

    /**
     * if filter in exclude filter set, return true
     * 
     * @param filterName
     * @return
     */
    public boolean isExcludeFilter(String filterName) {
        Set<String> excludeFilters = getExcludeFilters();
        return excludeFilters != null && excludeFilters.contains(filterName);
    }

    /**
     * reset the threadLocal context. Done at the end of the request.
     */
    public void reset() {
        THREAD_LOCAL.remove();
    }
}
