package com.jz.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.Result;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.common.enums.GeneralStatusTypeEnum;
import com.jz.dm.common.enums.auth.AuthModeEnum;
import com.jz.dm.common.util.RandomUtil;
import com.jz.dm.mapper.ApiAuthMapper;
import com.jz.dm.mapper.ApiInterfaceMapper;
import com.jz.dm.mapper.ApiOrgMapper;
import com.jz.dm.models.domian.ApiAuth;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.models.domian.ApiOrg;
import com.jz.dm.models.dto.AuthInfoDto;
import com.jz.dm.models.req.auth.*;
import com.jz.dm.models.resp.AuthUserResponse;
import com.jz.dm.service.AuthService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;

import static com.jz.dm.common.enums.auth.AuthModeEnum.fromTypeModeName;
import static com.jz.dm.common.enums.auth.AuthTypeEnum.fromTypeName;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: AuthServiceImpl
 * @DATE: 2020-12-23/16:48
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("authService")
@Slf4j
public class AuthServiceImpl implements AuthService {


    @Resource
    private ApiAuthMapper apiAuthMapper;
    @Resource
    private ApiInterfaceMapper apiInterfaceMapper;
    @Resource
    private ApiOrgMapper apiOrgMapper;

    /**
     * 认证信息列表
     *
     * @param req
     * @return
     */
    @Override
    public Result getAuthListInfo(AuthListInfoReq req) {
        IPage<AuthInfoDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<AuthInfoDto> query = new QueryWrapper<>();
        query.eq("au.is_deleted",0);
        query.orderByDesc("au.create_date");
        IPage<AuthInfoDto> listSelectApiAuth = apiAuthMapper.listSelectApiAuth(page, query);
        return Result.of_success(listSelectApiAuth);
    }

    /**
     * 认证信息详情
     *
     * @param req
     * @return
     */
    @Override
    public Result getAuthDetailInfo(AuthDetailInfoReq req) {
        return Result.of_success(apiAuthMapper.selectAuthDetail(req));
    }

    /**
     * 添加dmp认证信息
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class,propagation = Propagation.REQUIRES_NEW)
    public Result addDmpUserApiAuth(AuthUserApiReq req) {
        ApiInterface apiInterface = getInterface(req.getApiKey());
        if (null == apiInterface) {
            return Result.of_error("api授权信息不存在!");
        }
        ApiOrg apiOrg = apiOrgMapper.selectOne(new QueryWrapper<ApiOrg>().eq("org_code",req.getOrgCode()));
        if (null == apiOrg) {
            return Result.of_error("组织信息不存在!");
        }
        //授权验证
        if (StringUtils.isNotBlank(req.getAuthType())){
            if (null == fromTypeName(req.getAuthType())){
                return Result.of_error("授权类型不存在!");
            }
        }
        //验证授权方式
        if (StringUtils.isNotBlank(req.getAuthMode())){
            if (null == fromTypeModeName(req.getAuthMode())){
                return Result.of_error("授权方式不存在!");
            }
        }
        //认证类型为永久授权
        if (StringUtils.isNotBlank(req.getAuthMode()) &&
                !AuthModeEnum.PERMANENT_TIME_MODE.name().equals(req.getAuthMode())) {
            return Result.of_error("授权类型错误!");
        }
        if (StringUtils.isBlank(apiOrg.getOrgCode())){
            return Result.of_error("内部授权组织编码不能为空!!");
        }
        return saveAuthInfo(apiInterface,apiOrg,req,Constants.AUTH_INT);
    }

    /**
     * 添加商城用户认证信息
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class,propagation = Propagation.REQUIRES_NEW)
    public Result addMallUserApiAuth(AuthUserApiReq req) {
        ApiInterface apiInterface = getInterface(req.getApiKey());
        if (null == apiInterface) {
            return Result.of_error("api授权信息不存在!");
        }
        ApiOrg apiOrg = apiOrgMapper.selectOne(new QueryWrapper<ApiOrg>().eq("org_code",req.getOrgCode()));
        if (null == apiOrg) {
            return Result.of_error("组织信息不存在!");
        }
        //授权验证
        if (StringUtils.isNotBlank(req.getAuthType())){
            if (null == fromTypeName(req.getAuthType())){
                return Result.of_error("授权类型不存在!");
            }
        }
        //验证授权方式
        if (StringUtils.isNotBlank(req.getAuthMode())){
            if (null == fromTypeModeName(req.getAuthMode())){
                return Result.of_error("授权方式不存在!");
            }
        }
        //认证类型为永久授权
        if (StringUtils.isNotBlank(req.getAuthMode()) &&
                AuthModeEnum.PERMANENT_TIME_MODE.name().equals(req.getAuthMode())) {
            return Result.of_error("授权类型错误!");
        }
        //认证类型
        if (StringUtils.isNotBlank(req.getAuthMode()) &&
                AuthModeEnum.RECORD_TIME_MODE.name().equals(req.getAuthMode())){
            if (null == req.getValidStartTime() || null == req.getValidEndTime()) {
                return Result.of_error("开始或结束时间不能为空!");
            }
        }
        //验证用户id
        if (StringUtils.isBlank(req.getUserId())){
           return Result.of_error("外部授权用户Id不能为空");
        }
        return saveAuthInfo(apiInterface,apiOrg,req,Constants.AUTH_OUT);
    }

    /**
     * 保存授权信息
     * @param apiInterface
     * @param apiOrg
     * @param req
     * @param orgType
     * @return
     */
   private Result saveAuthInfo(ApiInterface apiInterface, ApiOrg apiOrg ,
                               AuthUserApiReq req,String orgType){
       AuthUserResponse response = new AuthUserResponse();
       ApiAuth apiAuth = new ApiAuth();
       apiAuth.setApiInterfaceId(apiInterface.getId());
       apiAuth.setApiOrgId(apiOrg.getId());
       BeanUtils.copyProperties(req, apiAuth);
       //商城只有外部授权
       String authCode ="";
       //内部组织
        if (StringUtils.isNotBlank(orgType) && Constants.AUTH_INT.equalsIgnoreCase(orgType)) {
            authCode = getAuthCode(apiOrg.getOrgCode(),"",Constants.AUTH_INT);
        //外部组织
        } else if (StringUtils.isNotBlank(orgType)
                && Constants.AUTH_OUT.equalsIgnoreCase(orgType)) {
            authCode = getAuthCode("", req.getUserId(),Constants.AUTH_OUT);
        }
       //生成盐值
       String salt = RandomUtil.getStringRandom(8);
       apiAuth.setAuthCode(authCode);//授权码
       apiAuth.setSalt(salt);//盐值
       apiAuth.setStatus(GeneralStatusTypeEnum.VALID.name());//授权状态
       apiAuth.setCreateUser("");
       response.setAuthCode(authCode);
       response.setSalt(salt);
       if (apiAuthMapper.insert(apiAuth) > 0) {
           return Result.of_success(ResultMsg.SUCCESS,response);
       }
       return Result.of_error(ResultMsg.INSERT_FAIL);
   }
    /**
     * 获取授权码
     * @param type
     * @param userId
     * @param apiOrgCode
     * @return
     */
    private String getAuthCode(String apiOrgCode,String userId,String type) {
        String authCode = "";
        authCode = RandomUtil.generateTokenCode(type, apiOrgCode, userId);
        QueryWrapper<ApiAuth> query = new QueryWrapper<>();
        query.eq("auth_code", authCode);
        ApiAuth auth = apiAuthMapper.selectOne(query);
        if (null != auth) {
            authCode = RandomUtil.generateTokenCode(type, apiOrgCode, userId);
        }
        return authCode;
    }

    /**
     * 修改用户认证信息
     *
     * @param req
     * @return
     */
    @Override
    public Result updateAuthInfo(AuthInfoUpdateReq req) {
        //TODO 认证后是否可以修改
        return null;
    }

    /**
     * 盐值重置
     *
     * @param req
     * @return
     */
    @Override
    public Result updateSaltInfo(SaltResetReq req) {
        AuthUserResponse response = new AuthUserResponse();
        ApiAuth apiAuth = apiAuthMapper.selectById(req.getId());
        if (null == apiAuth) {
            return Result.of_error("认证用户不存在");
        }
            String salt = RandomUtil.getStringRandom(8);
            UpdateWrapper<ApiAuth> update = new UpdateWrapper<>();
            update.set("salt", salt);
            update.set("create_date", new Date());
            update.set("create_user", "");
            update.eq("id", req.getId());
            if (apiAuthMapper.update(null, update) > 0) {
                response.setSalt(salt);
                return Result.of_success(ResultMsg.UPDATE_SUCCESS,response);
            }
        return Result.of_success(ResultMsg.UPDATE_FAIL);
    }

    /**
     * 获取API授权信息
     * @param authCode
     * @param apiId
     * @return
     */
    @Override
    public ApiAuth getAuthUser(String authCode,Long apiId) {
        QueryWrapper<ApiAuth> query = new QueryWrapper<>();
        query.eq("auth_code",authCode);
        query.eq("api_interface_id",apiId);
        query.eq("is_deleted",0);
        return apiAuthMapper.selectOne(query);
    }

    private ApiInterface getInterface(String apiKey) {
        QueryWrapper<ApiInterface> queryInface = new QueryWrapper<>();
        queryInface.last("where is_deleted=0 and api_key ='" + apiKey + "'");
        ApiInterface apiInterface = apiInterfaceMapper.selectOne(queryInface);
        if (null != apiInterface) {
            return apiInterface;
        }
        return null;
    }
}
