package com.jz.dm.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultCode;
import com.jz.common.constant.ResultMsg;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.HttpsUtils;
import com.jz.common.utils.RedisUtils;
import com.jz.common.utils.Result;
import com.jz.dm.common.constant.TagConstants;
import com.jz.dm.common.enums.GeneralStatusTypeEnum;
import com.jz.dm.common.enums.apiInterface.ApiStatusEnum;
import com.jz.dm.common.enums.produce.ProducerStatusTypeEnum;
import com.jz.dm.common.util.RandomUtil;
import com.jz.dm.mapper.*;
import com.jz.dm.models.domian.*;
import com.jz.dm.models.req.make.*;
import com.jz.dm.service.ProducerService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

import static com.jz.dm.common.enums.apiInterface.ApiStatusEnum.fromApiStatusName;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ProducerServiceImpl
 * @DATE: 2020-12-23/16:57
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("producerService")
@Slf4j
public class ProducerServiceImpl implements ProducerService {

    @Resource
    private ApiInterfaceMapper apiInterfaceMapper;
    @Resource
    private ApiSyncingDatasourceMapper apiSyncingDatasourceMapper;
    @Resource
    private ApiOpenApiEsFieldsMapper apiOpenApiEsFieldsMapper;
    @Resource
    private ApiOpenApiEsTagconfigMapper apiOpenApiEsTagconfigMapper;
    @Resource
    private ApiInterfaceCustomMapper apiInterfaceCustomMapper;
    @Resource
    private ApiInterfaceFileMapper apiInterfaceFileMapper;
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private HttpsUtils httpsUtils;

    @Value("${dmp.openapi.timeout.max}")
    private int maxTimeout;
    @Value("${dmp.openapi.timeout.default}")
    private int defaultTimeout;
    //@Value("${dmp.openapi.env.open:false}")
    //private boolean envOpen = false;
    @Value("${dmp.ignoreRedis}")
    protected boolean ignoreRedis;

    /**
     * 获取文件列表
     * @param projectId
     * @param pageSize
     * @param pageNum
     * @return
     */
    @Override
    public IPage<ApiInterfaceFile> getFileCatalog(Long projectId,String pageNum, String pageSize) {
        Integer pageN = Integer.valueOf(pageNum);
        Integer pageS = Integer.valueOf(pageSize);
        IPage<ApiInterfaceFile> page = new Page<>(pageN, pageS);
        QueryWrapper<ApiInterfaceFile> query = new QueryWrapper<>();
        query.eq("project_id",projectId);
        query.eq("is_deleted", 0);
        query.orderByDesc("create_date");
        return apiInterfaceFileMapper.selectPage(page, query);
    }

    /**
     * 制作自定义Api
     *
     * @param req
     * @return
     */
    @Override
    public Result addCustomApi(ApiInterfaceReq req) {
        return saveInterfaceAPi(req);
    }

    /**
     * 数据银行制作API/数据包
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addDataBankApiInfo(MakeDataBankApiReq req) {
        SysUserDto currentUser =  getLoginUser(RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        if (null == currentUser) {
            return Result.of_error(ResultMsg.USER_NOT_LOGIN);
        }
        List<ApiInterfaceFile> fileSource =
                apiInterfaceFileMapper.selectList(new QueryWrapper<ApiInterfaceFile>().eq("file_source", "2"));
        if (fileSource.size() == 0 || fileSource.size() >= 2) {
            return Result.of_error("文件夹信息异常!");
        }
        String apiKey ="";
        if (StringUtils.isNotBlank(req.getJoinType())
                && "10006".equals(req.getJoinType())) {//数据银行制作API
             apiKey = RandomUtil.getStringRandom(16);
            ApiInterface anInterface = new ApiInterface();
            anInterface.setCreateUser("");
            anInterface.setApiType("2");//数据银行制作api
            anInterface.setStatus(ProducerStatusTypeEnum.ISSUE.name());//发布
            anInterface.setApiKey(apiKey);//apiKey
            anInterface.setFileId(req.getFileId());
            BeanUtils.copyProperties(req, anInterface);
            int resIn = apiInterfaceMapper.insert(anInterface);
            ApiInterfaceCustom interfaceCustom = new ApiInterfaceCustom();
            interfaceCustom.setCreateUser("");
            interfaceCustom.setApiKey(apiKey);
            interfaceCustom.setApiInterfaceId(anInterface.getId());
            BeanUtils.copyProperties(req, interfaceCustom);
            int resOut = apiInterfaceCustomMapper.insert(interfaceCustom);
            if (resOut == 0 || resIn == 0) {
                throw ResponseException.of_error("保存信息失败!");
            }
            redisUtils.set(apiKey,req);
        } else if (StringUtils.isNotBlank(req.getJoinType())
                && "10007".equals(req.getJoinType())) {//数据银行制作数据包
            if (StringUtils.isBlank(req.getTargetUrl())) {//目标地址
                return Result.of_error("目标地址为空!");
            }
            ApiInterfaceReq apiInterfaceReq = new ApiInterfaceReq();
            apiInterfaceReq.setApiType("2");
            apiInterfaceReq.setTransMode(req.getApiProtocl());//请求方式:https http
            BeanUtils.copyProperties(req,apiInterfaceReq);
            Result result = saveInterfaceAPi(apiInterfaceReq);
            apiKey=(String) result.getData();
            redisUtils.set(apiKey,req);
        }
        return Result.of_success(ResultMsg.INSERT_SUCCESS,apiKey);
    }

    /**
     * Api制作(实时接入)
     *
     * @param req
     * @return
     */
    @Override
    public Result addRealCustomApi(MakeRealCustomApiReq req) {

        return null;
    }

    /**
     * Api制作(大数据查询/标签查询)
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addBigDataMakeApi(MakeBigDataApiReq req) {
        SysUserDto currentUser =  getLoginUser(RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        if (null == currentUser) {
            return Result.of_error(ResultMsg.USER_NOT_LOGIN);
        }
        QueryWrapper<ApiOpenApiEsFields> queryOpenApi = new QueryWrapper<>();
        queryOpenApi.eq("es_database", req.getEsDatabase());
        queryOpenApi.eq("es_table", req.getEsTable());
        queryOpenApi.eq("is_deleted", 0);
        List<ApiOpenApiEsFields> listOpenApiEsFields = apiOpenApiEsFieldsMapper.selectList(queryOpenApi);
        //如果存在先根据库，表删除，重新添加
        if (CollectionUtils.isNotEmpty(listOpenApiEsFields)) {
            Map paMap = new HashMap<>();
            apiOpenApiEsFieldsMapper.deleteByMap(paMap);
        }
        ArrayList<ApiOpenApiEsFields> listApiEsFields = new ArrayList<>();
        //保存字段表中的字段信息
        JSONArray json = JSONArray.parseArray(req.getTableFields());
        for (int i = 0; i < json.size(); i++) {
            JSONObject jsonObject = (JSONObject) json.get(i);
            ApiOpenApiEsFields fields = new ApiOpenApiEsFields();
            fields.setEsDatabase(req.getEsDatabase());
            fields.setEsTable(req.getEsTable());
            fields.setFieldName(jsonObject.get("name").toString());
            fields.setFieldType(jsonObject.get("type").toString());
            fields.setFieldDesc(jsonObject.get("desc").toString());
            fields.setCreateUser(currentUser.getUserName());
            listApiEsFields.add(fields);
        }
        //保存字段表中的字段信息
        if (CollectionUtils.isNotEmpty(listApiEsFields)) {
            apiOpenApiEsFieldsMapper.insetBatch(listApiEsFields);
        }
        ApiSyncingDatasource dataSource = apiSyncingDatasourceMapper.selectById(req.getEsDataSource());
        //api信息在redis中的key
        String redisValueOfKey = RandomUtil.getStringRandom(16);
        if (null != dataSource) {
            redisTemplate.opsForValue().set(TagConstants.OPEN_API_HANDLER_INFO_NAMESPACE + redisValueOfKey, req.getHandleType());
            //es相关信息
            //redisTemplate.opsForValue().set(TagConstants.OPEN_API_ESTAG_INFO_NAMESPACE + redisValueOfKey, req);
            //存入调用的基本ApiKey信息
            redisTemplate.opsForValue().set( redisValueOfKey, req);
            //esjdbc 连接地址 用于接口中调用获取es数据
            redisTemplate.opsForValue().set(TagConstants.OPEN_API_ESTAG_JDBC_INFO_NAMESPACE + redisValueOfKey, dataSource.getJdbcUrl());
        }
        int result = 0;
        ApiOpenApiEsTagconfig apiEsTagconfig = new ApiOpenApiEsTagconfig();
        if (null != req.getId() && "".equals(req.getId())) {//更新
            apiEsTagconfig.setUpdateDate(new Date());
            apiEsTagconfig.setUpdateUser(currentUser.getUserName());
            BeanUtils.copyProperties(req, apiEsTagconfig);
            result = apiOpenApiEsTagconfigMapper.updateById(apiEsTagconfig);
        } else {  //保存
            apiEsTagconfig.setCreateUser(currentUser.getUserName());
            BeanUtils.copyProperties(req, apiEsTagconfig);
            result = apiOpenApiEsTagconfigMapper.insert(apiEsTagconfig);
        }
        if (result <= 0) {
            return Result.of_error(ResultMsg.FAILURE);
        }
        return Result.of_success(ResultMsg.SUCCESS, req.getId());
    }

    /**
     * 保存api基本信息
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public Result saveInterfaceAPi(ApiInterfaceReq req) {
        SysUserDto currentUser =  getLoginUser(RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        if (null == currentUser) {
            return Result.of_error(ResultMsg.USER_NOT_LOGIN);
        }
        QueryWrapper<ApiInterfaceFile> query = new QueryWrapper<>();
        query.eq("project_id", req.getProjectId());
        query.eq("parent_id", req.getParentId());
        ApiInterfaceFile file = apiInterfaceFileMapper.selectOne(query);
        if (null == file) {
            //查询父类是否存在
            ApiInterfaceFile parentId = apiInterfaceFileMapper.selectOne(new QueryWrapper<ApiInterfaceFile>().eq("parent_id", req.getParentId()));
            if (null == parentId && "0".equals(req.getParentId())){
                file= new  ApiInterfaceFile();
                file.setProjectId(req.getProjectId());
                file.setFileSource("1");//dmp
                file.setParentId("0");
                file.setStatus(GeneralStatusTypeEnum.VALID.name());
                file.setCreateUser("");
                file.setRemark("创建文件夹");
                if ( apiInterfaceFileMapper.insert(file) == 0){
                    throw ResponseException.of_error("创建文件夹失败!");
                }
            }else {
                throw  ResponseException.of(ResultCode.FAILURE,"父类文件信息不存在!");
            }
        }
        //生成ApiKey
        String apiKey = RandomUtil.getStringRandom(16);
        ApiInterface apiInterface = new ApiInterface();
        apiInterface.setApiKey(apiKey);
        apiInterface.setApiProtocl(req.getTransMode());//传输方式 https http
        apiInterface.setStatus(ApiStatusEnum.ISSUE.name());//发布
        apiInterface.setFileId(file.getId());
        BeanUtils.copyProperties(req, apiInterface);
        apiInterface.setCreateUser(currentUser.getUserName());
        Long timeout = Long.parseLong(req.getTimeout());
        if (timeout == null || timeout == 0) {
            apiInterface.setTimeout(String.valueOf(defaultTimeout));
        } else if (timeout < 0) {
            return Result.of_error("接口超时时间最小0ms");
        } else if (timeout > maxTimeout) {
            return Result.of_success("接口超时时间最大5000ms");
        }
        if (apiInterfaceMapper.insert(apiInterface) == 0) {
            throw ResponseException.of_error("保存Api信息失败!");
        }
        return Result.of_success(ResultMsg.INSERT_SUCCESS,apiKey);
    }

    /**
     * 保存修改的api信息
     *
     * @param req
     * @return
     */
    @Override
    public Result saveUpdateAPiInfo(MakeApiSaveInfoReq req) {
        SysUserDto currentUser =  getLoginUser(RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        if (null == currentUser) {
            return Result.of_error(ResultMsg.USER_NOT_LOGIN);
        }
        ApiInterface apiInterface = apiInterfaceMapper.selectById(req.getId());
        if (null == apiInterface) {
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        if (StringUtils.isBlank(req.getType()) || "0".equals(req.getType())) {
            return Result.of_error("加密类型错误!");
        }
        apiInterface.setApiType(req.getApiType());
        apiInterface.setApiDesc(req.getApiDesc());
        apiInterface.setVersion(RandomUtil.generateVersion(apiInterface.getVersion()));//版本自动累加
        apiInterface.setSignType(req.getType());
        apiInterface.setApiProtocl(req.getTransMode());//传输方式 https http
        if (StringUtils.isNotBlank(req.getTargetUrl())) {
            apiInterface.setTargetUrl(req.getTargetUrl());
            if (StringUtils.isNotBlank(req.getTimeout())) {
                apiInterface.setTimeout(req.getTimeout());
            }
            apiInterface.setApiFunction(req.getApiFunction());
            if (StringUtils.isNotBlank(req.getStatus())){
                if (null == fromApiStatusName(req.getStatus())){
                  return Result.of_error("更新状态不存在!");
                }
            }
            apiInterface.setStatus(req.getStatus());
            if (StringUtils.isNotBlank(req.getStatus())) {
                boolean b = canChange(apiInterface.getStatus(), req.getStatus());
                if (!b) {
                    String errorMsg = String.format("api当前状态为%s不可变更为%s",
                            ApiStatusEnum.valueOf(apiInterface.getStatus()).getText(), ApiStatusEnum.valueOf(req.getStatus()).getText());
                    return Result.of_error(errorMsg);
                }
            }
            Long timeout = Long.parseLong(apiInterface.getTimeout());
            if (timeout == null || timeout == 0) {
                apiInterface.setTimeout(String.valueOf(defaultTimeout));
            } else if (timeout < 0) {
                return Result.of_error("接口超时时间最小0ms");
            } else if (timeout > maxTimeout) {
                return Result.of_success("接口超时时间最大5000ms");
            }
            apiInterface.setUpdateDate(new Date());
            apiInterface.setTimeout(req.getTimeout());
            apiInterface.setUpdateUser(currentUser.getUserName());
            int result = apiInterfaceMapper.updateById(apiInterface);
            if (result > 0) {
                return Result.of_success(ResultMsg.UPDATE_SUCCESS);
            }
        }
        return Result.of_success(ResultMsg.UPDATE_FAIL);
    }

    /**
     * 判断状态是否改变
     *
     * @param currentStatus
     * @param toStatus
     * @return
     */
    private boolean canChange(String currentStatus, String toStatus) {
        if (StringUtils.equals(currentStatus, toStatus)) {
            log.info("api的状态一直，crrentStatus:{},toStatus:{}", currentStatus, toStatus);
            return true;
        }
        //已经失效
        if (StringUtils.equals(currentStatus, ApiStatusEnum.EXPIRY.name())) {
            log.info("api的状态是：{}，不能修改为：{}", currentStatus, toStatus);
            return false;
        }
        //toStatus 草稿  其它状态不能变更为草稿
        if (StringUtils.equals(toStatus, ApiStatusEnum.DRAFT.name())) {
            log.info("api的状态是：{}[{}]，不能修改为：{}[{}]", currentStatus,
                    ApiStatusEnum.DRAFT.getText(), toStatus, ApiStatusEnum.DRAFT.getText());
            return false;
        }
        return true;
    }


    /**
     * 获取数据源配置
     *
     * @param req
     * @return
     */
    @Override
    public Result getSourceConfig(SourceConfigReq req) {
        if (StringUtils.isBlank(req.getApiKey())) {
            return Result.of_success("Api不能为空");
        }
        if (StringUtils.isNotBlank(req.getReleaseId())) {
            ApiInterface anInterface = apiInterfaceMapper.selectOne(new QueryWrapper<ApiInterface>().eq("api_key", req.getApiKey()));
            if (null != anInterface) {
                return Result.of_error("存在重复的Apikey");
            }
        } else {
            QueryWrapper<ApiInterface> query = new QueryWrapper<>();
            query.eq("api_key",req.getApiKey());
            query.eq("is_deleted",0);
            ApiInterface apiInterface = apiInterfaceMapper.selectOne(query);
            if (null != apiInterface) {
                return Result.of_error("存在重复的Apikey");
            }
        }
        return Result.of_success(ResultMsg.SUCCESS);
    }

    /**
     * 获取数据源库配置
     *
     * @param id
     * @return
     */
    @Override
    public Result getSourceWarehouseList(Long id) {
        ApiSyncingDatasource datasource = apiSyncingDatasourceMapper.selectById(id);
        if (null == datasource) {
            return Result.of_error("数据源不存在!");
        }
        JSONArray result = new JSONArray();
        log.info("查询es标签类下的索引地址信息:" + datasource.getJdbcUrl() + "/es/get/indicess");
        String resp = httpsUtils.doGet(datasource.getJdbcUrl() + "/es/get/indicess", null);
        if (resp != null) {
            log.info("返回索引信息集合:" + resp);
            JSONArray json = JSONArray.parseArray(resp);
            for (int i = 0; i < json.size(); i++) {
                String database = json.get(i).toString();
                /**
                 * 除去已点开头的索引返回前端
                 */
                if (!database.startsWith(".")) {
                    result.add(database);
                }
            }
            log.info("过滤后索引信息集合:" + result.toJSONString());
        }

        return Result.of_success(result);
    }

    /**
     * 获取某个索引下的type
     *
     * @param indices
     * @param id
     * @return
     */
    @Override
    public Result getIndicesTypes(String indices, Long id) {
        ApiSyncingDatasource datasource = apiSyncingDatasourceMapper.selectById(id);
        if (null == datasource) {
            return Result.of_error("数据源不存在!");
        }
        log.info("查询es标签类索引下的type的地址信息:" + datasource.getJdbcUrl() + "/es/get/indices/types");

        JSONObject param = new JSONObject();
        param.put("indices", indices);
        log.info("参数集合：" + param.toString());
        String resp = httpsUtils.submitPost(datasource.getJdbcUrl() + "/es/get/indices/types", param.toString());
        log.info("索引下type的信息集合:" + resp);
        return Result.of_success(resp);
    }

    /**
     * 获取某个索引 type的fields
     *
     * @param indices
     * @param type
     * @param id
     * @return
     */
    @Override
    public Result getIndicesTypesFields(String indices, String type, Long id) {
        ApiSyncingDatasource datasource = apiSyncingDatasourceMapper.selectById(id);
        if (null == datasource) {
            return Result.of_error("数据源不存在!");
        }
        log.info("查询es标签类洗typeFields地址信息:" + datasource.getJdbcUrl() + "/es/get/type/fields");
        HashMap<String, String> herders = new HashMap<>();
        herders.put("Content-Encoding", "UTF-8");
        herders.put("Content-Type", "application/json");
        JSONObject param = new JSONObject();
        param.put("indices", indices);
        param.put("type", type);
        String resp = null;
        try {
            resp = httpsUtils.submitPost(datasource.getJdbcUrl() + "/es/get/type/fields", param,herders);
            log.info("索引下type的fields信息集合:" + resp);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.of_success(resp);
    }

    /**
     * 获取当前登录用户
     * @param userType
     * @return
     */
    protected  SysUserDto getLoginUser(String userType){
        SysUserDto currentUser = (SysUserDto)
                redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
     if (null != currentUser){
         return currentUser;
     }
     return null;
    }
}
