package com.jz.dm.web.binder;


import com.jz.dm.web.annotation.ParamName;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerAdapter;
import org.springframework.web.servlet.mvc.method.annotation.ServletModelAttributeMethodProcessor;

import java.lang.reflect.Field;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 参数名注解绑定
 *
 */
public class ParamNameAnnotationResolver extends ServletModelAttributeMethodProcessor {

    @Autowired
    private RequestMappingHandlerAdapter requestMappingHandlerAdapter;

    /**
     * 参数名映射
     */
    private final Map<Class<?>, Map<String, String>> classToParamNamesMapping = new ConcurrentHashMap<Class<?>, Map<String, String>>();

    public ParamNameAnnotationResolver(boolean annotationNotRequired) {
        super(annotationNotRequired);
    }

    @Override
    protected void bindRequestParameters(WebDataBinder binder, NativeWebRequest nativeWebRequest) {
        Object target = binder.getTarget();
        Class<?> targetClass = target.getClass();
        if (!classToParamNamesMapping.containsKey(targetClass)) {
            Map<String, String> paramNamesMapping = getParamNamesMapping(targetClass);
            classToParamNamesMapping.put(targetClass, paramNamesMapping);
        }
        ParamNameAnnotationBinder paramNameDataBinder = new ParamNameAnnotationBinder(target,
            binder.getObjectName(), classToParamNamesMapping.get(targetClass));
        requestMappingHandlerAdapter.getWebBindingInitializer().initBinder(paramNameDataBinder,
            nativeWebRequest);
        super.bindRequestParameters(paramNameDataBinder, nativeWebRequest);
    }

    /**
     * 获取注解参数名和字段名映射
     * 
     * @param targetClass
     * @return
     */
    private static Map<String, String> getParamNamesMapping(Class<?> targetClass) {
        Field[] fields = targetClass.getDeclaredFields();
        Map<String, String> renameMap = new HashMap<String, String>();
        for (Field field : fields) {
            ParamName paramNameAnnotation = field.getAnnotation(ParamName.class);
            if (paramNameAnnotation != null && !paramNameAnnotation.value().isEmpty()) {
                renameMap.put(paramNameAnnotation.value(), field.getName());
            }
        }
        if (renameMap.isEmpty())
            return Collections.emptyMap();
        return renameMap;
    }
}