package com.jz.common.utils;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.ParseException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.jz.common.utils.HttpClientPool.getHttpClient;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.common.utils
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: HttpsUtils
 * @DATE: 2021-1-3/0:29
 * @DAY_NAME_SHORT: 周日
 * @Description:
 **/
@Slf4j
@Component
public class HttpsUtils {

    public static final String JSON_CONTENT_FORM = "application/json; charset=utf-8";
    public static final String CONTENT_FORM = "application/x-www-form-urlencoded;charset=UTF-8";

    /**
     * Post请求表单提交
     * @param url     请求路径
     * @param params  请求参数
     * @param headers 请求头
     * @return
     */
    public String submitPost(String url, String params, Map<String, String> headers) {
        CloseableHttpClient httpClient = getHttpClient();
        String body = null;
        CloseableHttpResponse response = null;
        try {
            HttpPost httpPost = new HttpPost(url);
            httpPost.setHeader("Content-type",JSON_CONTENT_FORM );
            if (null != headers && headers.size() > 0) {
                for (Map.Entry<String, String> e : headers.entrySet()) {
                    httpPost.addHeader(e.getKey(), e.getValue());
                }
            }
            if (StringUtils.isNotBlank(params)) {
                httpPost.setEntity(new StringEntity(params, Consts.UTF_8));
            }
            response = httpClient.execute(httpPost);
            body = getBody(response.getEntity());
            log.info("请求:" + url + "---->返回结果:" + body);
        } catch (IOException e) {
            log.error("请求:" + url + "--->异常", e);
        } finally {
            try {// 释放链接
                response.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return body;
    }

    /**
     * Post请求表单提交
     *
     * @param url     请求路径
     * @param params  请求参数
     * @return
     */
    public  String submitPost(String url, String params) {
        CloseableHttpClient httpClient = getHttpClient();
        String body = null;
        CloseableHttpResponse response = null;
        try {
            HttpPost httpPost = new HttpPost(url);
            httpPost.setHeader("Content-type",JSON_CONTENT_FORM );
            if (StringUtils.isNotBlank(params)) {
                httpPost.setEntity(new StringEntity(params, Consts.UTF_8));
            }
            response = httpClient.execute(httpPost);
            body = getBody(response.getEntity());
            log.info("请求:" + url + "---->返回结果:" + body);
        } catch (IOException e) {
            log.error("请求:" + url + "--->异常", e);
        } finally {
            try {// 释放链接
                response.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return body;
    }

    /**
     * 验证码实时校验
     * 无需cookie和referer时请求接口
     *
     * @param params 入参
     * @param baseUrl 发送请求的URL
     * @return URL 所代表远程资源的响应结果
     */
    public  String doGet(String baseUrl, Map<String, String> params) {
        CloseableHttpClient httpClient = getHttpClient();
        CloseableHttpResponse response = null;
        String body = null;
        try {
            List<String> mapList = new ArrayList<>();
            if (params != null) {
                for (Map.Entry<String, String> entry : params.entrySet()) {
                    mapList.add(entry.getKey() + "=" + entry.getValue());
                }
            }
            if (CollectionUtils.isNotEmpty(mapList)) {
                baseUrl = baseUrl + "?";
                String paramsStr = StringUtils.join(mapList, "&");
                baseUrl = baseUrl + paramsStr;
            }
            HttpGet httpGet = new HttpGet(baseUrl);
            response = httpClient.execute(httpGet);
            body = getBody(response.getEntity());
            log.info("获取请求结果为:" + body);
        } catch (Exception e) {
            log.warn("请求接口时-->" + baseUrl + "发生异常:", e);
        } finally {
            try {
                response.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return body;
    }
    /**
     * 验证码实时校验
     * 无需cookie和referer时请求接口
     * @param baseUrl 发送请求的URL
     * @param params 请求参数
     * @return URL 所代表远程资源的响应结果
     */
    public  String doGet(String baseUrl,Map<String, String> params, Map<String, String> headers) {
        CloseableHttpClient httpClient = getHttpClient();
        CloseableHttpResponse response = null;
        String body = null;
        try {
            List<String> mapList = new ArrayList<>();
            if (params != null) {
                for (Map.Entry<String, String> entry : params.entrySet()) {
                    mapList.add(entry.getKey() + "=" + entry.getValue());
                }
            }
            if (CollectionUtils.isNotEmpty(mapList)) {
                baseUrl = baseUrl + "?";
                String paramsStr = StringUtils.join(mapList, "&");
                baseUrl = baseUrl + paramsStr;
            }
            HttpGet httpGet = new HttpGet(baseUrl);
            if (headers != null && headers.size() > 0) {
                for (Map.Entry<String, String> e : headers.entrySet()) {
                    httpGet.addHeader(e.getKey(), e.getValue());
                }
            }
            response = httpClient.execute(httpGet);
            body = getBody(response.getEntity());
            log.info("获取请求结果为:" + body);
        } catch (Exception e) {
            log.warn("请求接口时-->" + baseUrl + "发生异常:", e);
        } finally {
            try {
                response.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return body;
    }
    /**
     * 获取图片二维码
     *
     * @param url
     * @throws IOException
     */
    public CloseableHttpClient doGetImg(String url) {
        CloseableHttpClient httpClient = getHttpClient();
        HttpGet httpGet = new HttpGet(url);
        try {
            httpClient.execute(httpGet);
        } catch (IOException e) {
            log.error("发起http request异常:{}", e);
        }
        return httpClient;
    }

    /**
     * 获取图片二维码
     *
     * @param url
     * @throws IOException
     */
    public HttpEntity doGetImg(String url, Map<String, String> headers) {
        CloseableHttpClient httpClient = getHttpClient();
        CloseableHttpResponse response = null;
        HttpGet httpGet = new HttpGet(url);
        if (headers != null && headers.size() > 0) {
            for (Map.Entry<String, String> e : headers.entrySet()) {
                httpGet.addHeader(e.getKey(), e.getValue());
            }
        }
        try {
            response = httpClient.execute(httpGet);
        } catch (IOException e) {
            log.error("发起http request异常:{}", e);
        }
        return response.getEntity();
    }

    public static String getBody(HttpEntity entity) throws ParseException, IOException {
        String body = "";
        if (entity != null) {
            // 按指定编码转换结果实体为String类型
            body = EntityUtils.toString(entity, Consts.UTF_8);
        }
        EntityUtils.consume(entity);
        return body;
    }
}
