package com.jz.dm.mall.moduls.controller.customer;

import com.aliyuncs.exceptions.ClientException;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.utils.Result;
import com.jz.common.utils.SMSUtils;
import com.jz.common.utils.StatusCode;
import com.jz.common.utils.ValidateCodeUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;


import java.util.concurrent.TimeUnit;

/**
 * @ClassName: 短信发送接口
 * @Author: Carl
 * @Date: 2020/12/2
 * @Version:
 */
@RestController
@RequestMapping("/validateCode")
@Api(tags = "短信发送--controller")
public class ValidateCodeController {

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 注册时发送的验证码
     * @param telephone
     * @return
     */
    @GetMapping(value = "/sendForLogin")
    @ApiOperation(value = "注册时发送的验证码")
    public Result sendForLogin(@RequestParam(value = "telephone") String telephone) {
        String key = RedisMessageConstant.SENDTYPE_LOGIN + "_" + telephone;
        // 通过手机号从redis获取验证码
        String codeInRedis = (String) redisTemplate.opsForValue().get(key);

        if (!StringUtils.isEmpty(codeInRedis)) {
            return new Result(false, "验证码已发送，请注意查收！", StatusCode.ERROR);
        }else {
            // 生成验证码
            String code = ValidateCodeUtils.generateValidateCode(6) + "";
            // 发送
            try {
                SMSUtils.sendShortMessage(SMSUtils.VALIDATE_CODE, telephone, code);
            } catch (ClientException e) {
                e.printStackTrace();
                return new Result(false, "验证码发送失败！", StatusCode.ERROR);
            }
            // 存入redis, 有效期为5分钟
            redisTemplate.opsForValue().set(key,code,5, TimeUnit.MINUTES);
            return new Result(true, "验证码发送成功！", StatusCode.OK);
        }
    }


    /**
     * 修改密码发送验证码
     * @param telephone
     * @return
     */
    @GetMapping("/send4Code")
    @ApiOperation(value = "修改密码发送验证码")
    public Result send4Code(@RequestParam(value = "telephone") String telephone) {
        String key = RedisMessageConstant.SENDTYPE_LOGIN + "_" + telephone;
        // 通过手机号从redis获取验证码
        String codeInRedis = (String) redisTemplate.opsForValue().get(key);

        if (!StringUtils.isEmpty(codeInRedis)) {
            return new Result(false, "验证码已发送，请注意查收！", StatusCode.ERROR);
        } else {
            // 生成验证码
            String code = ValidateCodeUtils.generateValidateCode(6) + "";
            // 发送
            try {
                SMSUtils.sendShortMessage(SMSUtils.VALIDATE_CODE, telephone, code);
            } catch (ClientException e) {
                e.printStackTrace();
                return new Result(false, "验证码发送失败！", StatusCode.ERROR);
            }
            // 存入redis, 有效期为5分钟
            redisTemplate.opsForValue().set(key, code, 5, TimeUnit.MINUTES);
            return new Result(true, "验证码发送成功！", StatusCode.OK);
        }
    }
}
