package com.jz.dm.mall.moduls.controller.goods;

import com.jz.common.base.BaseController;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.dm.mall.moduls.controller.goods.bean.dto.DataGoodsDto;
import com.jz.dm.mall.moduls.controller.goods.bean.dto.DataGoodsListDto;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DataGoodsListRequest;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DataGoodsRequest;
import com.jz.dm.mall.moduls.entity.DataGoods;
import com.jz.dm.mall.moduls.service.DataGoodsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;


/**
 * 数据商品(dataGoods)表控制层
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:30
 */
@RestController
@RequestMapping("dataGoods")
@Api(tags = "数据商品--controller")
public class DataGoodsController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private DataGoodsService dataGoodsService;

	/**
	 * 列表查询商品数据
	 * @param dataGoodsListRequest
	 * @param httpRequest
	 * @return
	 */
	@PostMapping(value = "/findList")
	@ApiOperation(value = "分页展示商品列表", notes = "首页点击分类展示商品列表")
	public PageInfoResponse<DataGoodsListDto> findByCategory(@RequestBody DataGoodsListRequest dataGoodsListRequest, HttpServletRequest httpRequest) {
		PageInfoResponse<DataGoodsListDto> pageInfo = new PageInfoResponse<DataGoodsListDto>();
		try {
			pageInfo = dataGoodsService.findList(dataGoodsListRequest, httpRequest);
		} catch (Exception e) {
			pageInfo.setMessage("查询失败");
			pageInfo.setCode(Constants.FAILURE_CODE);
			e.printStackTrace();
		}
		return pageInfo;
	}

	
	/**主键查询数据商品
	 * @param dataGoodsRequest
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/view")
	public BaseBeanResponse<DataGoodsDto> view(@RequestBody DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest){
		BaseBeanResponse<DataGoodsDto> baseBeanResponse = new BaseBeanResponse<DataGoodsDto>();
		try {
			baseBeanResponse = dataGoodsService.findById(dataGoodsRequest, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("请求失败");
			baseBeanResponse.setCode(Constants.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
	/**删除数据商品
	 * @param dataGoodsRequest
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/delete")
	public BaseResponse delete(@RequestBody DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest){
		BaseResponse baseResponse = new BaseResponse();
		try {
			baseResponse = dataGoodsService.deleteById(dataGoodsRequest, httpRequest);
		} catch (Exception e) {
			baseResponse.setMessage("删除失败");
			baseResponse.setCode(Constants.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseResponse;
	}
	
	/**新增数据商品
	 * @param dataGoods
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/add")
	public BaseBeanResponse<DataGoods> add(@RequestBody DataGoods dataGoods, HttpServletRequest httpRequest){
		BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<DataGoods>();
		try {
			baseBeanResponse = dataGoodsService.add(dataGoods, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("新增失败");
			baseBeanResponse.setCode(Constants.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
	/**修改数据商品
	 * @param dataGoods
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/edit")
	public BaseBeanResponse<DataGoods> edit(@RequestBody DataGoods dataGoods, HttpServletRequest httpRequest){
		BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<DataGoods>();
		try {
			baseBeanResponse = dataGoodsService.edit(dataGoods, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("修改失败");
			baseBeanResponse.setCode(Constants.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}

}