package com.jz.dm.mall.moduls.controller.order;

import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.order.bean.OrderDto;
import com.jz.dm.mall.moduls.controller.order.bean.OrderRequest;
import com.jz.dm.mall.moduls.service.OrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;


/**
 * 订单表(TOrder)表控制层
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:41
 */
@RestController
@RequestMapping("/order")
@Api(tags = "订单管理--controller")
public class OrderController {
    /**
     * 服务对象
     */
    @Autowired
    private OrderService orderService;

    /**
     * 订单管理列表（分页查询）
     *
     * @author Bellamy
     */
    @PostMapping(value = "/list")
    @ApiOperation(value = "订单管理列表（分页查询）", notes = "订单列表（分页查询）")
    public PageInfoResponse<OrderDto> queryPageList(@RequestBody OrderRequest orderRequest, HttpServletRequest req) throws Exception {
        PageInfoResponse<OrderDto> pageInfo = new PageInfoResponse<OrderDto>();
        try {
            pageInfo = orderService.findList(orderRequest, req);
        } catch (Exception e) {
            pageInfo.setMessage("查询失败");
            pageInfo.setCode(Constants.FAILURE_CODE);
            e.printStackTrace();
        }
        return pageInfo;
    }

    /**
     * 根据订单主键查询，订单详情
     *
     * @param orderId
     * @author Bellamy
     */
    @GetMapping(value = "/orderDetail")
    @ApiOperation(value = "订单详情", notes = "订单详情")
    public Result<OrderDto> getOrderDetail(@RequestParam(value = "orderId") String orderId, HttpServletRequest req) throws Exception {
        Result<OrderDto> result = new Result<OrderDto>();
        if (StringUtils.isNotEmpty(orderId)) {
            OrderDto orderDto = orderService.getOrderDetail(orderId);
            result.setData(orderDto);
        } else {
            result.setMessage("参数为不能为空！");
        }
        return result;
    }

    /**
     * 根据订单主键查询，订单详情
     *
     * @param orderId,dataGoodsId
     * @author Bellamy
     */
    @GetMapping(value = "/apiInterfaceDetail")
    @ApiOperation(value = "订单详情---接口文档", notes = "接口文档")
    public Result<Map> getApiInterfaceDetail(@RequestParam(value = "orderId") String orderId, @RequestParam(value = "dataGoodsId") String dataGoodsId, HttpServletRequest req) throws Exception {
        Result<Map> result = new Result<Map>();
        Map params = new HashMap();

        if (StringUtils.isNotEmpty(dataGoodsId)) {
            params.put("dataGoodsId", dataGoodsId);
        }
        if (StringUtils.isNotEmpty(orderId)) {
            params.put("orderId", orderId);
        }
        //查询接口文档 和参数信息
        Map dataGoodsApi = orderService.getApiInterfaceDetail(params);
        result.setData(dataGoodsApi);
        return result;
    }


}