package com.jz.dm.mall.moduls.service.impl;

import com.jz.common.base.CurrentUser;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.constant.ResultCode;
import com.jz.common.constant.ResultMsg;
import com.jz.common.entity.Department;
import com.jz.common.enums.UserTypeEnum;
import com.jz.common.exception.ResponseException;
import com.jz.dm.mall.moduls.controller.company.bean.CompanyUpdateReq;
import com.jz.dm.mall.moduls.controller.company.dto.CompanyInfoDto;
import com.jz.dm.mall.moduls.entity.FinanceCustomerAssets;
import com.jz.dm.mall.moduls.entity.MallCustomer;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.company.bean.CompanyAddReq;
import com.jz.dm.mall.moduls.mapper.DepartmentDao;
import com.jz.dm.mall.moduls.mapper.FinanceCustomerAssetsDao;
import com.jz.dm.mall.moduls.mapper.MallCustomerDao;
import com.jz.dm.mall.moduls.service.CompanyAuthService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.mall.moduls.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: CompanyAuthServiceImpl
 * @USER: key
 * @DATE: 2020-12-2/16:35
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("companyAuthService")
@Slf4j
public class CompanyAuthServiceImpl implements CompanyAuthService {

    @Resource
    private DepartmentDao departmentDao;
    @Resource
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;
    @Resource
    private MallCustomerDao mallCustomerDao;

    /**
     * 添加企业认证
     * @param request
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addCompanyData(CompanyAddReq req, HttpServletRequest request) {
        //获取当前用户ID判断当前用户是否已经关联企业
        //MallCustomerApiDto currentUser = CurrentUser.getCurrentUser(request);
        MallCustomerApiDto currentUser = new MallCustomerApiDto();
        currentUser.setCustomerId(1L);
        currentUser.setCustomerName("张胜男");
        if (null == currentUser) {
            return Result.of_error(ResultMsg.USER_NOT_EXIST);
        }
        MallCustomer mallCustomer = mallCustomerDao.findById(currentUser.getCustomerId());
        if (null == mallCustomer) {
            return Result.of_error(ResultMsg.USER_NOT_EXIST);
        }
        if (null != mallCustomer.getDepartmentId()) {
            return Result.of_error("用户已关联企业,请勿重复操作");
        }
        req.setLoginId(mallCustomer.getCustomerId());
        req.setLoginName(currentUser.getCustomerName());
        if (StringUtils.isNotBlank(req.getDepartmentName()) ||
            StringUtils.isNotBlank(req.getUnifiedCreditCode())) { //企业名称 && 统一社会编码
            Department department = departmentDao.selectDepartmentData(req);
            if (null != department) {
                return Result.of_error("企业用户信息已存在");
            }
        }
        insetCompany(req);
        return Result.of_success(ResultMsg.SUCCESS);
    }

    /**
     * 查询企业认证详情
     * @param request
     * @param type
     * @return
     */
    @Override
    public Result selectCompany(String type,HttpServletRequest request) {
        // 查询用户判断用户是否认证
        MallCustomer currentUser = getCurrentUser(CurrentUser.getCustomerId(request));
        if (null == currentUser) {
            return Result.error("当前用户信息不存在");
        }
        if (StringUtils.equals(type, "STATUS")) {//状态
            String auditStatus = departmentDao.selectCompanyStatus(currentUser.getDepartmentId());
            return Result.of_success(auditStatus);
        } else if (StringUtils.equals(type, "DETAIL")) {//详情
            CompanyInfoDto department = departmentDao.selectCompanyDetail(currentUser.getDepartmentId());
            return Result.of_success(department);
        }
        return Result.of_error("查询企业信息异常");
    }

    /**
     * 获取登录用户信息
     *
     * @param customerId
     * @return
     */
    private MallCustomer getCurrentUser(Long customerId) {
        try {
            MallCustomer mallCustomer = mallCustomerDao.findById(customerId);
            if (null != mallCustomer) {
                return mallCustomer;
            }
        } catch (Exception e) {
            log.error("用户:" + customerId + "信息不存在~~~~~~~~~~");
        }
        return null;
    }

    /**
     * 更新企业信息
     * @param request
     * @param req
     * @return
     */
    @Override
    public Result updateCompanyInfo(CompanyUpdateReq req,HttpServletRequest request) {
        Long customerId = CurrentUser.getCustomerId(request);
        MallCustomer currentUser = getCurrentUser(customerId);
        if (null == currentUser) {
            return Result.of_error(ResultMsg.USER_NOT_EXIST);
        }
        if (!req.getDepartmentId().equals(customerId)) {
            return Result.of_error("更新企业用户信息与绑定企业信息不一致");
        }
        Department department = new Department();
        department.setUptTime(new Date());
        department.setUptPerson(req.getLoginName());
        BeanUtils.copyProperties(req, department);
        if (departmentDao.updateById(department) != 1) {
            throw ResponseException.of_error("更新企业信息失败");
        }
        return Result.of_success(ResultMsg.UPDATE_SUCCESS);
    }

    /**
     * 校验用户是否已经企业认证
     * @param request
     * @return
     */
    @Override
    public Result checkCompanyInfo(HttpServletRequest request) {
        MallCustomer currentUser = getCurrentUser(CurrentUser.getCustomerId(request));
        if (null == currentUser) {
            return Result.of_success(ResultMsg.USER_NOT_EXIST);
        }
        CompanyInfoDto companyInfoDto = departmentDao.selectCompanyDetail(currentUser.getDepartmentId());
        if (null != companyInfoDto && "02".equals(companyInfoDto.getAuditStatus())) {
            return Result.of_success("用户已进行企业认证");
        }
        return Result.of_success("用户未进行企业认证");
    }

    /**
     * 保存企业信息
     *
     * @param req
     */
    private void insetCompany(CompanyAddReq req) {
        //保存企业信息
        Department departmentInset = new Department();
        departmentInset.setAuditStatus("01");//待审核
        departmentInset.setCreTime(new Date());
        departmentInset.setCrePerson(req.getLoginName());
        BeanUtils.copyProperties(req, departmentInset);
        if (departmentDao.insertDepart(departmentInset) != 1) {
            throw ResponseException.of_error("保存企业信息失败");
        }
        //初始化用户资产表
        FinanceCustomerAssets finance = new FinanceCustomerAssets();
        finance.setDepartmentId(departmentInset.getDepartmentId());//企业ID
        finance.setCreTime(new Date());
        finance.setCrePerson(req.getLoginName());
        if (financeCustomerAssetsDao.insertFinanceCustome(finance) != 1) {
            throw ResponseException.of_error("初始化用户资产失败");
        }
        //更新用户企业信息
        //MallCustomer mallCustomer = new MallCustomer();
        //mallCustomer.setCustomerId(req.getLoginId());//用户ID
        //mallCustomer.setDepartmentId(departmentInset.getDepartmentId());
        //mallCustomer.setUptTime(new Date());
        //mallCustomer.setUptPerson(req.getLoginName());
        //if (mallCustomerDao.updateById(mallCustomer) != 1) {
        //    throw ResponseException.of_error("更新用户企业信息失败");
        //}
    }
}
