package com.jz.dm.mall.moduls.service.impl;

import com.jz.common.constant.ResultCode;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.Result;
import com.jz.common.utils.StatusCode;
import com.jz.dm.mall.moduls.controller.goods.bean.dto.DataDetailsDto;
import com.jz.dm.mall.moduls.controller.goods.bean.dto.DataGoodsApiDto;

import com.jz.dm.mall.moduls.mapper.DataGoodsApiDao;
import com.jz.dm.mall.moduls.service.DataGoodsApiService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * api商品(TDataGoodsApi)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:31
 */
@Service("dataGoodsApiService")
public class DataGoodsApiServiceImpl implements DataGoodsApiService {
    @Autowired
    private DataGoodsApiDao tDataGoodsApiDao;


    /**
     * 根据主键查询商品api详情
     *
     * @param id
     * @return
     */
    @Override
    public Result selectById(Long id) {
        List<DataGoodsApiDto> dataGoodsApiDto = tDataGoodsApiDao.selectById(id);
        if (dataGoodsApiDto.size() == 0) {
            return Result.error(StatusCode.ERROR, "查询异常！");
        }
        List<DataGoodsApiDto> list1 = new ArrayList<>();
        List<DataGoodsApiDto> list2 = new ArrayList<>();
        List<DataGoodsApiDto> list3 = new ArrayList<>();
        Map<String, Object> map = new HashMap<>();

        for (DataGoodsApiDto goodsApiDto : dataGoodsApiDto) {
            if (goodsApiDto.getParamsDiffNum().equals("01")) {
                list1.add(goodsApiDto);
            }
            if (goodsApiDto.getParamsDiffNum().equals("02")) {
                list2.add(goodsApiDto);
            }
            if (goodsApiDto.getParamsDiffNum().equals("03")) {
                list3.add(goodsApiDto);
            }
            if (!StringUtils.isEmpty(goodsApiDto.getRequestExample())) {
                map.put("requestExample", goodsApiDto.getRequestExample()); // 请求示例
            }
            if (!StringUtils.isEmpty(goodsApiDto.getReturnDataExample())) {
                map.put("returnDataExample", goodsApiDto.getReturnDataExample()); // 响应示例
            }
        }
        // 商品详情
        DataGoodsApiDto goods = dataGoodsApiDto.get(0);
        DataDetailsDto data = new DataDetailsDto();
        data.setSupplierName(goods.getSupplierName());
        data.setCategoryName(goods.getCategoryName());
        data.setPriceType(goods.getPriceType());
        data.setYearType(goods.getYearType());
        data.setSeasonType(goods.getSeasonType());
        data.setMonthType(goods.getMonthType());
        data.setSecondType(goods.getSecondType());
        data.setApiUrl(goods.getApiUrl());
        data.setApiMethod(goods.getApiMethod());
        data.setRequestType(goods.getRequestType());
        data.setReturnType(goods.getReturnType());
        data.setDataPicture(goods.getDataPicture());
        data.setDataDesc(goods.getDataDesc());
        data.setDataPrice(goods.getDataPrice());
        map.put("dataInfo", data);

        // 商品参数
        map.put("pubParam", list1); // 公共参数
        map.put("reqParam", list2); // 请求参数
        map.put("resParam", list3); // 响应参数

        Result result = new Result();
        result.setData(map);
        return Result.of_success(ResultMsg.SUCCESS, result);
    }


}