package com.jz.dm.mall.moduls.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.base.CurrentUser;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.SessionUtils;
import com.jz.dm.mall.moduls.entity.PlatformLog;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.log.bean.LogInfoQueryReq;
import com.jz.dm.mall.moduls.entity.MallCustomer;
import com.jz.dm.mall.moduls.mapper.MallCustomerDao;
import com.jz.dm.mall.moduls.mapper.PlatformLogDao;
import com.jz.dm.mall.moduls.service.LogInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.mall.moduls.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: LogInfoServiceImpl
 * @USER: key
 * @DATE: 2020-12-3/19:00
 * @DAY_NAME_SHORT: 周四
 * @Description:
 **/
@Slf4j
@Service("logInfoService")
public class LogInfoServiceImpl implements LogInfoService {

    @Resource
    private PlatformLogDao platformLogDao;
    @Resource
    private MallCustomerDao mallCustomerDao;

    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 获取商城用户日志列表
     * @param req
     * @param request
     * @return
     */
    @Override
    public  PageInfoResponse<PlatformLog> getMallUserLogInfo(LogInfoQueryReq req, HttpServletRequest request) {
        PageInfoResponse<PlatformLog> pageInfoResponse = new PageInfoResponse<>();
        //MallCustomerApiDto currentUser = CurrentUser.getCurrentUser(request);
        MallCustomerApiDto currentUser = (MallCustomerApiDto) redisTemplate.opsForValue().get("user_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
//        System.out.println(currentUser.getCustomerName());
        if (null == currentUser) {
            pageInfoResponse.setCode(Constants.SUCCESS_CODE);
            pageInfoResponse.setMessage(ResultMsg.USER_NOT_EXIST.getMsg());
            return pageInfoResponse;
        }
        //MallCustomerApiDto currentUser =new MallCustomerApiDto();
        //currentUser.setCustomerId(1L);
        MallCustomer mallCustomer = mallCustomerDao.findById(currentUser.getCustomerId());
        if (null == mallCustomer) {
            pageInfoResponse.setCode(Constants.SUCCESS_CODE);

            pageInfoResponse.setMessage(ResultMsg.USER_NOT_EXIST.getMsg());
            return pageInfoResponse;
        }
        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        List<PlatformLog> list = platformLogDao.listMallLogInfo(mallCustomer.getCustomerId());
        PageInfo<PlatformLog> pageInfo = new PageInfo<>(list);
        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage(ResultMsg.SUCCESS.getMsg());
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 获取商城用户日志信息详情
     * @param platformLogId
     * @return
     */
    @Override
    public Result getMallUserLogDetail(Long platformLogId) {
        PlatformLog platformLog = platformLogDao.findByPlatformId(platformLogId);
        if (null == platformLog){
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        return Result.of_success(platformLog);
    }
}
