package com.jz.dm.mall.moduls.service.impl;


import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.enums.UserTypeEnum;

import com.jz.common.utils.DateUtils;
import com.jz.common.utils.SessionUtils;
import com.jz.dm.mall.moduls.controller.customer.bean.CustomerDto;
import com.jz.dm.mall.moduls.controller.customer.bean.req.CustomerRequest;
import com.jz.dm.mall.moduls.entity.MallCustomer;
import com.jz.dm.mall.moduls.mapper.MallCustomerDao;
import com.jz.dm.mall.moduls.service.MallCustomerService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;


/**
 * 商城用户(TMallCustomer)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
@Service("mallCustomerService")
public class MallCustomerServiceImpl implements MallCustomerService {
    @Autowired
    private MallCustomerDao tMallCustomerDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 通过用户账号进行查询
     *
     * @param username
     * @return
     */
    @Override
    public CustomerDto selectByAccount(String username, HttpServletRequest request) {
        CustomerDto mallCustomer = tMallCustomerDao.selectByAccount(username);

//        String customer = JSON.toJSONString(mallCustomer);
        if (mallCustomer != null) {
            MallCustomerApiDto mallCustomerApiDto = new MallCustomerApiDto();
            // 赋值
            mallCustomerApiDto.setCustomerId(mallCustomer.getCustomerId());
            mallCustomerApiDto.setDepartmentId(mallCustomer.getDepartmentId());
            mallCustomerApiDto.setUserTypeEnum(UserTypeEnum.COMPANY_ROLE);
            mallCustomerApiDto.setCustomerAccount(mallCustomer.getCustomerAccount());
            mallCustomerApiDto.setCustomerName(mallCustomer.getCustomerName());
            mallCustomerApiDto.setAssetsId(mallCustomer.getAssetsId());
            // 存入到session
            request.getSession().setAttribute("mallCustomer", mallCustomerApiDto);

        // 存入到redis
            redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomerApiDto, 3, TimeUnit.DAYS);
    }
        return mallCustomer;
    }

    /**
     * 通过手机号进行查询
     *
     * @param username
     * @return
     */
    @Override
    public CustomerDto selectByPhone(String username, HttpServletRequest request) {
        CustomerDto mallCustomer = tMallCustomerDao.selectByPhone(username);
        if (mallCustomer != null) {
            MallCustomerApiDto mallCustomerApiDto = new MallCustomerApiDto();
            // 赋值
            mallCustomerApiDto.setCustomerId(mallCustomer.getCustomerId());
            mallCustomerApiDto.setDepartmentId(mallCustomer.getDepartmentId());
            mallCustomerApiDto.setUserTypeEnum(UserTypeEnum.COMPANY_ROLE);
            mallCustomerApiDto.setCustomerAccount(mallCustomer.getCustomerAccount());
            mallCustomerApiDto.setCustomerName(mallCustomer.getCustomerName());
            mallCustomerApiDto.setAssetsId(mallCustomer.getAssetsId());
            // 存入到session
            request.getSession().setAttribute("mallCustomer", mallCustomerApiDto);

            // 存入到redis
            redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomerApiDto, 3, TimeUnit.DAYS);
        }
        return mallCustomer;
    }

    /**
     * 注册账号
     *
     * @param
     */
    @Override
    public void saveCustomer(CustomerRequest customerRequest) {
        MallCustomer mallCustomer = new MallCustomer();
        // 获取验证码
        String vailCode = customerRequest.getVailCode();
        String username = customerRequest.getUsername();
        String telephone = customerRequest.getTelephone();

        // 从redis获取验证码
        String key = RedisMessageConstant.SENDTYPE_LOGIN + "_" + telephone;
//        String key = "18179617425";
        String codeInRedis = (String) redisTemplate.opsForValue().get(key);
//        String codeInRedis = "147826";
        if (codeInRedis.equals(vailCode)) {
            mallCustomer.setCustomerAccount(username);
            mallCustomer.setPassword(customerRequest.getPassword());
            mallCustomer.setCustomerPhone(telephone);
            mallCustomer.setCreTime(DateUtils.getToday());
            mallCustomer.setRegisterTime(DateUtils.getToday());
            tMallCustomerDao.saveCustomer(mallCustomer);
        }
    }

    /**
     * 通过手机号查询是否已注册
     *
     * @param telephone@return
     */
    @Override
    public MallCustomer selectByPhone(String telephone) {
        MallCustomer mallCustomer = tMallCustomerDao.selectByPhoneExist(telephone);
        if (mallCustomer != null) {
            return mallCustomer;
        }
        return null;
    }

    /**
     * 修改密码
     *
     * @param customerId
     * @param password
     */
    @Override
    public void updatePassword(Long customerId, String password) {
        MallCustomer mallCustomer = tMallCustomerDao.findById(customerId);
        if (mallCustomer != null) {
            Map map = new HashMap();
            map.put("password", password);
            map.put("customerId", customerId);
            tMallCustomerDao.updatePassword(map);
        }
    }

    /**
     * 通过id查找用户信息
     *
     * @param customerId
     * @return
     */
    @Override
    public MallCustomer selectByUser(Long customerId) {
        MallCustomer mallCustomer = tMallCustomerDao.findById(customerId);
        return mallCustomer;
    }

//    /**
//     * 忘记密码
//     *
//     * @param customer
//     */
//    @Override
//    public void upPassword(CustomerRequest customer) {
//        // 通过手机号判断是否注册过
//
//    }


}