package com.jz.dm.mall.moduls.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.auth.AuthModeEnum;
import com.jz.common.enums.auth.AuthTypeEnum;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.NumberUtils;
import com.jz.common.utils.RestTemplateUtils;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.pay.dto.PayDto;
import com.jz.dm.mall.moduls.controller.pay.req.AuthMallUserApiReq;

import com.jz.dm.mall.moduls.entity.*;
import com.jz.dm.mall.moduls.mapper.*;

import com.jz.dm.mall.moduls.service.PayService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.client.RestTemplate;


import java.math.BigDecimal;
import java.util.Date;

/**
 * @ClassName:
 * @Author: Carl
 * @Date: 2020/12/31
 * @Version:
 */
@Service("payService")
public class PayServiceImpl implements PayService {


    @Autowired
    private OrderDao orderDao;

    @Autowired
    private OrderPaymentDao orderPaymentDao;

    @Autowired
    private OrderGoodsInfoDao goodsInfoDao;

    @Autowired
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;

    @Autowired
    private FinanceTradeFlowDao financeTradeFlowDao;

    @Autowired
    private RestTemplate restTemplate;

    @Value("${domain.apigateway}")
    private String apiGateway;

    /**
     * 新增订单
     *
     * @param payDto
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result goodsPay(PayDto payDto) throws Exception{
        // 新增订单主表
        Order order = new Order();
        order.setOrderNumber(NumberUtils.getOrderNumber(payDto.getCustomerId() + ""));
        order.setCustomerId(payDto.getCustomerId());
        order.setOrderMoney(payDto.getDataPrice());
        order.setOrderStatus("02");
        order.setPaymentMoney(payDto.getDataPrice());
        order.setOrderTime(DateUtils.getToday());
        order.setPaymentTime(order.getOrderTime());
        order.setPaymentMethod("01");
//        order.setSellerId();
//        order.setDistrictMoney();
        order.setTakeEffectTime(order.getOrderTime());
        //处理生效期
        Date time = getTime(payDto);
        if (time != null) {
            order.setInvalidTime(time);
        }
        order.setPurchaseMethod(payDto.getPurchaseMethod());
        order.setPriceType(payDto.getPriceType());
        order.setCrePerson(payDto.getCustomerId() + "");
        order.setCreTime(DateUtils.getToday());
        orderDao.addOrder(order);

        // 新增订单支付表/订单商品表
        Long orderId = order.getOrderId();
        if (orderId != null) {
            OrderPayment orderPayment = new OrderPayment();
//            orderPayment.setSellerId();
            orderPayment.setPaymentMoney(payDto.getDataPrice());
            orderPayment.setPaymentTime(DateUtils.getToday());
            orderPayment.setPaymentMethod("01");
//            orderPayment.setDistrictMoney();
            orderPayment.setPaymentStatus(order.getOrderStatus());
            orderPayment.setPurchaseMethod(payDto.getPurchaseMethod());
            orderPayment.setCreTime(DateUtils.getToday());
            orderPayment.setCrePerson(payDto.getCustomerId()+"");

            // 订单商品表
            OrderGoodsInfo goodsInfo = new OrderGoodsInfo();
            goodsInfo.setOrderId(orderId);
            goodsInfo.setDataGoodsId(payDto.getDataGoodsId());
            goodsInfo.setCreTime(DateUtils.getToday());
            goodsInfo.setCrePerson(payDto.getCustomerId()+"");

            // 新增
            orderPaymentDao.addOrderPayment(orderPayment);
            goodsInfoDao.addGoodsInfo(goodsInfo);
        }

        // 获取企业资产信息
//        payDto.getAssetsId()
        FinanceCustomerAssets assets = financeCustomerAssetsDao.findById(Long.valueOf("34"));
        // 可用金额减去支付金额
        BigDecimal bigDecimal = new BigDecimal(assets.getUseMoney()+ "");
        BigDecimal bigDecima2 = new BigDecimal(payDto.getDataPrice()+ "");
        if (bigDecimal.compareTo(bigDecima2) == -1){
            return Result.error("金额不足！请充值！");
        }
        BigDecimal subtract = bigDecimal.subtract(bigDecima2);


        AuthMallUserApiReq apiReq = new AuthMallUserApiReq();
        if ("04".equals(payDto.getPurchaseMethod())) {
            apiReq.setAuthMode(AuthModeEnum.RECORD_TIME_MODE);
        }else {
            apiReq.setAuthMode(AuthModeEnum.RECORD_TIME_MODE);
        }
        apiReq.setApiKey(payDto.getApiKey());
        apiReq.setAuthType(AuthTypeEnum.DATA_BANK_AUTH);
        apiReq.setOrgCode("WAISJYX40E3e"); // TODO
        apiReq.setOrgType("OUT");
        apiReq.setUserId(payDto.getCustomerId() + "");
        apiReq.setValidStartTime(order.getTakeEffectTime());
        apiReq.setValidEndTime(order.getInvalidTime());
        String post = RestTemplateUtils.post(apiGateway, JSON.toJSONString(apiReq));

        JSONObject params = JSONObject.parseObject(post);
        if (!params.get("code").equals(200) || StringUtils.isEmpty(params.get("data"))){
            return Result.of_error("授权失败！");
        }
        assets.setUseMoney(subtract);
        assets.setTotalMoney(subtract);
        assets.setUptTime(DateUtils.getToday());
        assets.setUptPerson(payDto.getCustomerId()+ "");
        financeCustomerAssetsDao.updateAssets(assets);

        // 新增记录表
        Long assetsId = assets.getAssetsId();
        if (assetsId!= null) {
            FinanceTradeFlow flow = new FinanceTradeFlow();
            flow.setAssetsId(assetsId);
            flow.setOrderId(orderId);
            flow.setTradeFlowNumber(NumberUtils.getDealNum());// 交易流水编号
            flow.setTradeMoney(payDto.getDataPrice());
            flow.setTradeType("03");
            flow.setCreTime(DateUtils.getToday());
            flow.setCrePerson(payDto.getCustomerId() + "");
            flow.setTradeFlowNumber(NumberUtils.getDealNum());
            financeTradeFlowDao.addFlow(flow);
        }
        return Result.of_success(ResultMsg.SUCCESS,params);
    }



    /**
     * 制作有效时间
     * @param payDto
     * @return
     */
    public static Date getTime(PayDto payDto) {
        String purchaseMethod = payDto.getPurchaseMethod();
        Date now = DateUtils.getToday();
        if ("01".equals(purchaseMethod)) {
            Date nowAddYear = DateUtils.getAfterYear(now);
            return nowAddYear;
        }else if ("02".equals(purchaseMethod)){
            Date nowAddSeason = DateUtils.getAfterSeason(now);
            return nowAddSeason;
        }else if("03".equals(purchaseMethod)) {
            Date nowAddMonth = DateUtils.getAfterMonth(now);
            return nowAddMonth;
        }else {
            return null;
        }
    }
}
