package com.jz.manage.moduls.controller.finance;

import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.BaseController;
import com.jz.manage.moduls.controller.finance.platForm.BalanceAuditRequest;
import com.jz.manage.moduls.controller.finance.platForm.BalanceInfoDto;
import com.jz.manage.moduls.controller.finance.platForm.BalanceListDto;
import com.jz.manage.moduls.controller.finance.platForm.BalanceListRequest;
import com.jz.manage.moduls.service.FinanceCustomerBalanceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * 企业客户充值记录(TFinanceCustomerBalance)表控制层
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:39
 */
@RestController
@RequestMapping("financeCustomerBalance")
@Api(tags = "平台--利润中心")
public class FinanceCustomerBalanceController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private FinanceCustomerBalanceService financeCustomerBalanceService;

    /**
     * 充值管理--分页列表
     *
     * @return
     * @author Bellamy
     * @since 2020-12-03
     */
    @PostMapping(value = "/cashOutList")
    @ApiOperation(value = "充值管理--分页列表", notes = "分页列表")
    public PageInfoResponse<BalanceListDto> getFinanceCashOutList(@RequestBody BalanceListRequest balanceListRequest, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<BalanceListDto> pageInfo = new PageInfoResponse<BalanceListDto>();
        Map map = new HashMap();
        //从session中获取平台用户id
        //SysUserDto user = (SysUserDto) SessionUtils.getUserCurrent(requset,"sysUser");
        //map.put("userId", user.getUserId());
        try {
            pageInfo = financeCustomerBalanceService.findListBalance(balanceListRequest, httpRequest);
        } catch (Exception e) {
            pageInfo.setMessage("查询失败");
            pageInfo.setCode(Constants.FAILURE_CODE);
            e.printStackTrace();
        }
        return pageInfo;
    }

    /**
     * 充值管理--充值详情
     *
     * @return
     * @author Bellamy
     * @since 2020-12-06
     */
    @GetMapping(value = "/balanceInfoById")
    @ApiOperation(value = "充值管理--充值详情", notes = "充值详情")
    public Result<BalanceInfoDto> getFinanceBalanceInfoById(@RequestParam(value = "customerBalanceId") String customerBalanceId, HttpServletRequest httpRequest) throws Exception {
        Result<BalanceInfoDto> result = new Result<>();
        if (StringUtils.isEmpty(customerBalanceId)) {
            result.setMsg("参数为空！");
            result.setSuccess(false);
            return result;
        }
        BalanceInfoDto dto = financeCustomerBalanceService.queryBalanceByid(customerBalanceId);
        result.setData(dto);
        return result;
    }

    /**
     * 充值管理--充值审核
     *
     * @return
     * @author Bellamy
     * @since 2020-12-08
     */
    @PostMapping(value = "/balanceInfoAudit")
    @ApiOperation(value = "充值管理--充值审核", notes = "充值审核")
    public Result<Object> getBalanceInfoAudit(@RequestBody BalanceAuditRequest balanceAuditRequest, HttpServletRequest httpRequest) throws Exception {
        Result<Object> result = new Result<>();
        if (StringUtils.isEmpty(balanceAuditRequest.getCustomerBalanceId())) {
            return Result.error("参数不能为空！");
        }
        if (StringUtils.isEmpty(balanceAuditRequest.getAuditStatus())) {
            return Result.error("参数不能为空！");
        }
        Result<Object> dto = financeCustomerBalanceService.uptBalanceInfoAuditByid(balanceAuditRequest);
        return dto;
    }
}