package com.jz.manage.moduls.controller.sys;

import com.aliyuncs.exceptions.ClientException;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.utils.Result;
import com.jz.common.utils.SMSUtils;
import com.jz.common.utils.ValidateCodeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.concurrent.TimeUnit;

/**
 * @ClassName: 手机发送验证码
 * @Author: Carl
 * @Date: 2020/12/1
 * @Version:
 */
@RestController
@RequestMapping("/ValidateCode")
public class ValidateCodeController {

    @Autowired
    private RedisTemplate redisTemplate;

    @PostMapping(value = "/sendForCode")
    public Result sendForCode(String telephone) {
        String key = RedisMessageConstant.SENDTYPE_LOGIN + "_" + telephone;
        // 通过手机号从redis获取验证码
        String codeInRedis = (String) redisTemplate.opsForValue().get(key);

        if (!StringUtils.isEmpty(codeInRedis)) {
            return new Result(false, "验证码已发送，请注意查收！");
        }else {
            // 生成验证码
            String code = ValidateCodeUtils.generateValidateCode(6) + "";
            // 发送
            try {
                SMSUtils.sendShortMessage(SMSUtils.VALIDATE_CODE, telephone, code);
            } catch (ClientException e) {
                e.printStackTrace();
                return new Result(false, "验证码发送失败！");
            }
            // 存入redis, 有效期为5分钟
            redisTemplate.opsForValue().set(key,code,5, TimeUnit.MINUTES);
            return new Result(true, "验证码发送成功！");
        }
    }
}
