package com.jz.manage.moduls.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.Constants;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.entity.DataGoods;
import com.jz.common.utils.*;

import com.jz.manage.moduls.controller.goods.bean.dto.DataGoodsDto;
import com.jz.manage.moduls.controller.goods.bean.dto.DataGoodsListDto;
import com.jz.manage.moduls.controller.goods.bean.request.DataApiUpReq;
import com.jz.manage.moduls.controller.goods.bean.request.DataGoodsListRequest;
import com.jz.manage.moduls.controller.goods.bean.request.DataGoodsRequest;
import com.jz.manage.moduls.controller.goods.bean.request.MakeDataBankApiReq;
import com.jz.manage.moduls.entity.DataGoodsApi;
import com.jz.manage.moduls.entity.DataGoodsApiParams;
import com.jz.manage.moduls.mapper.DataGoodsApiDao;
import com.jz.manage.moduls.mapper.DataGoodsApiParamsDao;
import com.jz.manage.moduls.mapper.DataGoodsDao;
import com.jz.manage.moduls.service.DataGoodsService;


import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.CrossOrigin;


/**
 * 数据商品(DataGoods)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:30
 */
@Service("dataGoodsService")
@CrossOrigin
public class DataGoodsServiceImpl implements DataGoodsService {
    @Autowired
    private DataGoodsDao dataGoodsDao;

    @Autowired
    private DataGoodsApiParamsDao apiParamsDao;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private DataGoodsApiDao tDataGoodsApiDao;

    @Autowired
    private HttpsUtils httpsUtils;

    @Value("${domain.apigateway}")
    private String apiGateway;

    /**
     * 条件分页查询数据列表
     *
     * @param goodsListRequest
     * @param httpRequest
     * @return
     * @throws Exception
     */
    @Override
    public PageInfoResponse<DataGoodsListDto> findList(DataGoodsListRequest goodsListRequest, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DataGoodsListDto> pageInfoResponse = new PageInfoResponse<>();

        Map<String, Object> param = new HashMap<>();
        //数据商品id
        if (goodsListRequest.getDataGoodsId() != null) {
            param.put("dataGoodsId", goodsListRequest.getDataGoodsId());
        }
        //商品分类id（行业）
        if (goodsListRequest.getCategoryId() != null) {
            param.put("categoryId", goodsListRequest.getCategoryId());
        }
        //数据商品名称
        if (!StringUtils.isEmpty(goodsListRequest.getDataName())) {
            param.put("dataName", goodsListRequest.getDataName());
        }
        //数据类型：01api，02数据包
        if (!StringUtils.isEmpty(goodsListRequest.getDataType())) {
            param.put("dataType", goodsListRequest.getDataType());
        }
        //数据状态：01预售，02在售中，03下架，04上架
        if (!StringUtils.isEmpty(goodsListRequest.getDataStatus())) {
            param.put("dataStatus", goodsListRequest.getDataStatus());
        }

        PageHelper.startPage(goodsListRequest.getPageNum(), goodsListRequest.getPageSize());
        List<DataGoodsListDto> list = dataGoodsDao.findList(param);
        PageInfo<DataGoodsListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }


    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#add(com.ycxc.vmts.entity.DataGoods, javax.servlet.http.HttpServletRequest)
     */
//	@Override
//	@Transactional(rollbackFor=Exception.class)
//	public BaseBeanResponse<DataGoods> add(DataGoods dataGoods, HttpServletRequest httpRequest) throws Exception {
//		BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<>();
//
//		/*$*seDataGoodsIdCodeReplace*$*/
//		//创建人
//		//dataGoods.setCreateUser(getHttpRequestUserId(httpRequest));
//		//创建时间
//		dataGoods.setCreTime(new Date());
//
//		dataGoodsDao.insertSelective(dataGoods);
//
//		baseBeanResponse.setCode(Constants.SUCCESS_CODE);
//		baseBeanResponse.setMessage("新增成功");
//		baseBeanResponse.setData(dataGoods);
//
//		return baseBeanResponse;
//	}

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#edit(com.ycxc.vmts.entity.DataGoods, javax.servlet.http.HttpServletRequest)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseBeanResponse<DataGoods> edit(DataGoods dataGoods, HttpServletRequest httpRequest) throws Exception {
        BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<>();

        //修改人
        //dataGoods.setUpdateUser(getHttpRequestUserId(httpRequest));
        //修改时间
        dataGoods.setUptTime(new Date());

        dataGoodsDao.updateByPrimaryKeySelective(dataGoods);

        baseBeanResponse.setCode(Constants.SUCCESS_CODE);
        baseBeanResponse.setMessage("修改成功");
        baseBeanResponse.setData(dataGoods);

        return baseBeanResponse;
    }

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#findById(com.ycxc.vmts.controller.bean.DataGoodsRequest, javax.servlet.http.HttpServletRequest)
     */
    @Override
    public BaseBeanResponse<DataGoodsDto> findById(DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest)
            throws Exception {
        BaseBeanResponse<DataGoodsDto> baseBeanResponse = new BaseBeanResponse<>();

        DataGoodsDto DataGoodsDto = dataGoodsDao.findById(dataGoodsRequest.getDataGoodsId());

        baseBeanResponse.setCode(Constants.SUCCESS_CODE);
        baseBeanResponse.setMessage("查询成功");
        baseBeanResponse.setData(DataGoodsDto);

        return baseBeanResponse;
    }

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#deleteById(com.ycxc.vmts.controller.bean.DataGoodsRequest, javax.servlet.http.HttpServletRequest)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse deleteById(DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest)
            throws Exception {
        BaseResponse baseResponse = new BaseResponse();

        dataGoodsDao.deleteByPrimaryKey(dataGoodsRequest.getDataGoodsId());

        baseResponse.setCode(Constants.SUCCESS_CODE);
        baseResponse.setMessage("删除成功");

        return baseResponse;
    }

    /**
     * 商品上下架
     * @param goodsId
     * @param statusId
     */
    @Override
    public Result editStatus(Long goodsId, Long statusId) {
        // 获取用户信息
        SysUserDto sysUserDto = (SysUserDto) redisTemplate.opsForValue().get("user_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);

        // 获取商品的信息
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("data_goods_id", goodsId);
        DataGoods dataGoods = dataGoodsDao.selectOne(queryWrapper);
        // 上架
        if ("01".equals(statusId)) {
            dataGoods.setDataStatus("04");
            queryWrapper.eq("data_status", dataGoods.getDataStatus());
            queryWrapper.eq("upt_time", DateUtils.getToday());
            queryWrapper.eq("upt_person", sysUserDto.getUserId());
            dataGoodsDao.update(dataGoods, queryWrapper);
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }

        // 下架
        if ("02".equals(statusId)) {
            dataGoods.setDataStatus("03");
            queryWrapper.eq("data_status", dataGoods.getDataStatus());
            queryWrapper.eq("upt_time", DateUtils.getToday());
            queryWrapper.eq("upt_person", sysUserDto.getUserId());
            dataGoodsDao.update(dataGoods, queryWrapper);
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }
        return Result.of_error(ResultMsg.UPDATE_FAIL);
    }
//
//    @Override
//    public Result getJoinType(String parentKey) {
//        Map headers = new HashMap<>();
//        headers.put("Content-type","application/x-www-form-urlencoded;charset=UTF-8");
//        String url = "http://127.0.0.1:8088/api/direction/getApiTypeList";
//        String s = httpsUtils.submitPost(url,parentKey , headers);
//        System.out.println(s);
//        return null;
//    }


    /**
     * 自营数据-上传api
     *
     * @param dataApiUpReq
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result uploadApi(DataApiUpReq dataApiUpReq) throws Exception  {
        // 获取当前用户
        SysUserDto user = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);

        if (dataApiUpReq == null) {
            return Result.error("上传失败！");
        }
        if (!StringUtils.isNotEmpty(dataApiUpReq.getReturnType())) {
            return Result.error("输出类型不能为空");
        }
        if (!StringUtils.isNotEmpty(dataApiUpReq.getJoinType())) {
            return Result.error("接入类型不存在！");
        }
        if (!StringUtils.isNotEmpty(dataApiUpReq.getApiProtocl())) {
            return Result.error("传输方式不能为空！");
        }
        if (!StringUtils.isNotEmpty(dataApiUpReq.getEncryMode())) {
            return Result.error("加密方式不能为空！");
        }
        // 调用getway获取apikey
        MakeDataBankApiReq apiReq = new MakeDataBankApiReq();
        apiReq.setApiType(dataApiUpReq.getApiType());
        apiReq.setApiDesc(dataApiUpReq.getApiDesc());
        apiReq.setApiProtocl(dataApiUpReq.getApiProtocl());
        apiReq.setSignType(dataApiUpReq.getEncryMode());
        apiReq.setTargetUrl(dataApiUpReq.getTargetUrl());
        apiReq.setApiFunction(dataApiUpReq.getApiFunction());
        apiReq.setApiUrl(dataApiUpReq.getApiUrl());
        apiReq.setJoinType(dataApiUpReq.getJoinType());
        apiReq.setOutputType(dataApiUpReq.getReturnType());
//        apiReq.setPage(); // TODO
        apiReq.setFileId(1L);
//        apiReq.setTimeout(dataApiUpReq.getOverTime() + "");
        for (DataGoodsApiParams params : dataApiUpReq.getInboxParam()) {
            apiReq.setInboxParam(JSON.toJSONString(params));
        }
        for (DataGoodsApiParams reqHeader : dataApiUpReq.getReqHeaders()) {
            apiReq.setReqHeaders(JSON.toJSONString(reqHeader));
        }
        for (DataGoodsApiParams params : dataApiUpReq.getRequestParam()) {
            apiReq.setRequestParam(JSON.toJSONString(params));
        }
        for (DataGoodsApiParams params : dataApiUpReq.getResponseParam()) {
            apiReq.setResponseParam(JSON.toJSONString(params));
        }
        for (DataGoodsApiParams params : dataApiUpReq.getRespCode()) {
            apiReq.setResponseParam(JSON.toJSONString(params));
        }
        apiReq.setApiExample(dataApiUpReq.getReturnDataExample());

//        String url= "http://47.115.53.1:8088/api/producer/addDataBankApiInfo";
//        String url= "http://127.0.0.1:8088/api/producer/addDataBankApiInfo";

        String post = httpsUtils.submitPost(apiGateway, JSON.toJSONString(apiReq));
        System.out.println(post);
        JSONObject result = JSON.parseObject(post);
        if (!result.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(result.get("data"))){
            return Result.of_error("制作失败！");
        }
        //封装参数
        DataGoods dataGoods = new DataGoods();
        dataGoods.setApiOpenId(dataApiUpReq.getApiOpenId());
        dataGoods.setCreTime(DateUtils.getToday());
        dataGoods.setCrePerson(user.getUserId() + "");
        dataGoods.setAuditTime(DateUtils.getToday());
        dataGoods.setAuditPerson(user.getUserId() + "");
        dataGoods.setAuditStatus("02");
        dataGoods.setDataStatus("02");
        BeanUtils.copyProperties(dataApiUpReq, dataGoods);
        dataGoodsDao.insert(dataGoods);

        // 赋值api
        Long dataGoodsId = dataGoods.getDataGoodsId();
        if (dataGoodsId != null) {
            DataGoodsApi dataGoodsApi = new DataGoodsApi();

            dataGoodsApi.setDataGoodsId(dataGoodsId);
            dataGoodsApi.setCreTime(DateUtils.getToday());
            dataGoodsApi.setCrePerson(user.getUserId() + "");
            dataGoodsApi.setEncryMode("MD5");
            dataGoodsApi.setTransMode("http");
            dataGoodsApi.setVersionNumber("v1.0");
            dataGoodsApi.setApiKey(result.get("data").toString());
            BeanUtils.copyProperties(dataApiUpReq, dataGoodsApi);
            tDataGoodsApiDao.insert(dataGoodsApi);

            // 赋值params
            Long goodsApi = dataGoodsApi.getGoodsApi();
            if (goodsApi != null) {
                // 公共参数
                List<DataGoodsApiParams> inboxParam = dataApiUpReq.getInboxParam();
                if (inboxParam != null || inboxParam.size() != 0) {
                    for (DataGoodsApiParams params : inboxParam) {
                        params.setGoodsApi(goodsApi);
                        params.setParamsDiff("01");
                        params.setCreTime(DateUtils.getToday());
                        params.setCrePerson(user.getUserId() + "");
                        BeanUtils.copyProperties(dataApiUpReq, params);
                        apiParamsDao.insert(params);
                    }
                }
                // 请求参数
                List<DataGoodsApiParams> requestParam = dataApiUpReq.getRequestParam();
                if (requestParam != null || requestParam.size() != 0) {
                    for (DataGoodsApiParams params : requestParam) {
                        params.setGoodsApi(goodsApi);
                        params.setParamsDiff("02");
                        params.setCreTime(DateUtils.getToday());
                        params.setCrePerson(user.getUserId() + "");
                        BeanUtils.copyProperties(dataApiUpReq, params);
                        apiParamsDao.insert(params);
                    }
                }
                // 响应参数
                List<DataGoodsApiParams> responseParam = dataApiUpReq.getResponseParam();
                if (responseParam != null || responseParam.size() != 0) {
                    for (DataGoodsApiParams params : responseParam) {
                        params.setGoodsApi(goodsApi);
                        params.setParamsDiff("03");
                        params.setCreTime(DateUtils.getToday());
                        params.setCrePerson(user.getUserId() + "");
                        BeanUtils.copyProperties(dataApiUpReq, params);
                        apiParamsDao.insert(params);
                    }
                }
                // 请求头参数
                List<DataGoodsApiParams> paramsList = dataApiUpReq.getReqHeaders();
                if (paramsList != null || paramsList.size() != 0) {
                    for (DataGoodsApiParams params : paramsList) {
                        params.setGoodsApi(goodsApi);
                        params.setParamsDiff("04");
                        params.setCreTime(DateUtils.getToday());
                        params.setCrePerson(user.getUserId() + "");
                        BeanUtils.copyProperties(dataApiUpReq, params);
                        apiParamsDao.insert(params);
                    }
                }
                // 状态码参数
                List<DataGoodsApiParams> respCode = dataApiUpReq.getRespCode();
                if (respCode != null || respCode.size() != 0) {
                    for (DataGoodsApiParams params : paramsList) {
                        params.setGoodsApi(goodsApi);
                        params.setParamsDiff("05");
                        params.setCreTime(DateUtils.getToday());
                        params.setCrePerson(user.getUserId() + "");
                        BeanUtils.copyProperties(dataApiUpReq, params);
                        apiParamsDao.insert(params);
                    }
                }
            }
        }
        return Result.of_success(ResultMsg.SUCCESS);
    }
}