package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.enums.AuditStatusEnum;
import com.jz.common.enums.TradeFlowEnum;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.finance.platForm.BalanceAuditRequest;
import com.jz.manage.moduls.controller.finance.platForm.BalanceInfoDto;
import com.jz.manage.moduls.controller.finance.platForm.BalanceListDto;
import com.jz.manage.moduls.controller.finance.platForm.BalanceListRequest;
import com.jz.manage.moduls.entity.FinanceCustomerAssets;
import com.jz.manage.moduls.entity.FinanceCustomerBalance;
import com.jz.manage.moduls.entity.FinanceTradeFlow;
import com.jz.manage.moduls.mapper.FinanceCustomerAssetsDao;
import com.jz.manage.moduls.mapper.FinanceCustomerBalanceDao;
import com.jz.manage.moduls.mapper.FinanceTradeFlowDao;
import com.jz.manage.moduls.service.FinanceCustomerBalanceService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 企业客户充值记录(TFinanceCustomerBalance)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:39
 */
@Service("financeCustomerBalanceService")
@Transactional
public class FinanceCustomerBalanceServiceImpl implements FinanceCustomerBalanceService {

    private static final Logger logger = LoggerFactory.getLogger(FinanceCustomerBalanceServiceImpl.class);

    @Autowired
    private FinanceCustomerBalanceDao financeCustomerBalanceDao;

    @Autowired
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;

    @Autowired
    private FinanceTradeFlowDao financeTradeFlowDao;

    @Override
    public PageInfoResponse<BalanceListDto> findListBalance(BalanceListRequest balanceListRequest, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<BalanceListDto> pageInfoResponse = new PageInfoResponse<>();

        Map param = new HashMap();
        //从session中获取平台用户id
        //SysUserDto user = (SysUserDto) SessionUtils.getUserCurrent(requset,"sysUser");
        //param.put("userId", user.getUserId());
        //充值描述
        if (StringUtils.isNotEmpty(balanceListRequest.getRemark())) {
            param.put("remark", balanceListRequest.getRemark());
        }
        //企业名
        if (StringUtils.isNotEmpty(balanceListRequest.getDepartmentName())) {
            param.put("departmentName", balanceListRequest.getDepartmentName());
        }
        //充值状态
        if (StringUtils.isNotEmpty(balanceListRequest.getAuditStatus())) {
            param.put("auditStatus", balanceListRequest.getAuditStatus());
        }
        //充值时间
        if (StringUtils.isNotEmpty(balanceListRequest.getBalanceTime())) {
            String balanceTime = balanceListRequest.getBalanceTime();
            String[] times = balanceTime.split("-");
            param.put("startTime", times[0] + "00:00:00");
            param.put("endTime", times[1] + "23:59:59");
        }
        PageHelper.startPage(balanceListRequest.getPageNum(), balanceListRequest.getPageSize());
        List<BalanceListDto> list = financeCustomerBalanceDao.findListBalance(param);
        PageInfo<BalanceListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    @Override
    public BalanceInfoDto queryBalanceByid(String customerBalanceId) throws Exception {
        return financeCustomerBalanceDao.queryBalanceByid(customerBalanceId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> uptBalanceInfoAuditByid(BalanceAuditRequest balanceAuditRequest) throws Exception {
        //1.修改充值记录状态
        FinanceCustomerBalance balance = new FinanceCustomerBalance();
        balance.setAuditStatus(balanceAuditRequest.getAuditStatus());
        balance.setRejectReason(balanceAuditRequest.getRejectReason());
        balance.setCustomerBalanceId(Long.valueOf(balanceAuditRequest.getCustomerBalanceId()));
        //balance.setUptPerson();
        balance.setUptTime(new Date());
        int len = financeCustomerBalanceDao.updateById(balance);
        if (len > 0) {
            logger.info("###########################修改充值审核状态成功####################################");
            //2.查询出充值金额  和 账户id
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("customer_balance_id", balanceAuditRequest.getCustomerBalanceId());
            wrapper.eq("del_flag", "N");
            FinanceCustomerBalance financeCustomerBalance = financeCustomerBalanceDao.selectOne(wrapper);
            Long assetsId = financeCustomerBalance.getAssetsId(); //企业用户资产id
            String balanceMoney = financeCustomerBalance.getBalanceMoney().toString(); //充值金额
            BigDecimal balanceMoneyObj = new BigDecimal(balanceMoney);

            //3.查询资产金额
            QueryWrapper assetsWrapper = new QueryWrapper();
            assetsWrapper.eq("assets_id", assetsId);
            wrapper.eq("del_flag", "N");
            FinanceCustomerAssets financeCustomerAssets = financeCustomerAssetsDao.selectOne(assetsWrapper);
            String frozenMoney = financeCustomerAssets.getFrozenMoney().toString(); // 冻结金额
            BigDecimal frozenMoneyObj = new BigDecimal(frozenMoney);
            frozenMoneyObj = frozenMoneyObj.subtract(balanceMoneyObj);

            if (AuditStatusEnum.YSH.getValue().equals(balanceAuditRequest.getAuditStatus())) {
                //审核成功 给总金额和可用余额加上充值金额 ，冻结减去
                String useMoney = financeCustomerAssets.getUseMoney().toString(); // 可用金额
                BigDecimal useMoneyObj = new BigDecimal(useMoney);
                useMoneyObj = useMoneyObj.add(balanceMoneyObj);

                String totalMoney = financeCustomerAssets.getTotalMoney().toString(); //总金额
                BigDecimal totalMoneyObj = new BigDecimal(totalMoney);
                totalMoneyObj = totalMoneyObj.add(balanceMoneyObj);

                //4.更新 企业资产
                FinanceCustomerAssets saveAssets = new FinanceCustomerAssets();
                saveAssets.setAssetsId(assetsId);
                saveAssets.setUseMoney(useMoneyObj);
                saveAssets.setTotalMoney(totalMoneyObj);
                saveAssets.setFrozenMoney(frozenMoneyObj);
                saveAssets.setUptTime(new Date());
                //saveAssets.setUptPerson();
                financeCustomerAssetsDao.updateById(saveAssets);
                logger.info("###########################充值审核通过---更新企业资产成功####################################");

                //添加流水记录
                FinanceTradeFlow financeTradeFlow = new FinanceTradeFlow();
                financeTradeFlow.setCreTime(new Date());
                financeTradeFlow.setTradeMoney(balanceMoneyObj);
                financeTradeFlow.setAssetsId(assetsId);
                financeTradeFlow.setTradeType(TradeFlowEnum.CZ.getValue());
                financeTradeFlow.setTradeFlowNumber("123132");
                financeTradeFlow.setBalanceMoney(useMoneyObj);
                financeTradeFlow.setDelFlag("N");
                //financeTradeFlow.setCashOutId();
                //financeTradeFlow.setCrePerson();
                financeTradeFlowDao.insert(financeTradeFlow);
                logger.info("#####################商城提现流水记录添加成功#####################");
            }
            if (AuditStatusEnum.WTG.getValue().equals(balanceAuditRequest.getAuditStatus())) {
                //4.审核未通过 更新 企业资产
                FinanceCustomerAssets saveAssets = new FinanceCustomerAssets();
                saveAssets.setAssetsId(assetsId);
                saveAssets.setFrozenMoney(frozenMoneyObj);
                saveAssets.setUptTime(new Date());
                //saveAssets.setUptPerson();
                financeCustomerAssetsDao.updateById(saveAssets);
                logger.info("###########################充值审核未通过--更新企业资产成功####################################");
            }
            return Result.ok();
        } else {
            return Result.error("提交失败！");
        }
    }
}